from django.test import TestCase

from .models import (
    Event,
    Movie,
    Package,
    PackageNullFK,
    Person,
    Screening,
    ScreeningNullFK,
)


# These are tests for #16715. The basic scheme is always the same: 3 models
# with 2 relations. The first relation may be null, while the second is
# non-nullable. In some cases, Django would pick the wrong join type for the
# second relation, resulting in missing objects in the queryset.
#
#   Model A
#   | (Relation A/B : nullable)
#   Model B
#   | (Relation B/C : non-nullable)
#   Model C
#
# Because of the possibility of NULL rows resulting from the LEFT OUTER JOIN
# between Model A and Model B (i.e. instances of A without reference to B),
# the second join must also be LEFT OUTER JOIN, so that we do not ignore
# instances of A that do not reference B.
#
# Relation A/B can either be an explicit foreign key or an implicit reverse
# relation such as introduced by one-to-one relations (through multi-table
# inheritance).
class NestedForeignKeysTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.director = Person.objects.create(name="Terry Gilliam / Terry Jones")
        cls.movie = Movie.objects.create(
            title="Monty Python and the Holy Grail", director=cls.director
        )

    # This test failed in #16715 because in some cases INNER JOIN was selected
    # for the second foreign key relation instead of LEFT OUTER JOIN.
    def test_inheritance(self):
        Event.objects.create()
        Screening.objects.create(movie=self.movie)

        self.assertEqual(len(Event.objects.all()), 2)
        self.assertEqual(len(Event.objects.select_related("screening")), 2)
        # This failed.
        self.assertEqual(len(Event.objects.select_related("screening__movie")), 2)

        self.assertEqual(len(Event.objects.values()), 2)
        self.assertEqual(len(Event.objects.values("screening__pk")), 2)
        self.assertEqual(len(Event.objects.values("screening__movie__pk")), 2)
        self.assertEqual(len(Event.objects.values("screening__movie__title")), 2)
        # This failed.
        self.assertEqual(
            len(
                Event.objects.values("screening__movie__pk", "screening__movie__title")
            ),
            2,
        )

        # Simple filter/exclude queries for good measure.
        self.assertEqual(Event.objects.filter(screening__movie=self.movie).count(), 1)
        self.assertEqual(Event.objects.exclude(screening__movie=self.movie).count(), 1)

    # These all work because the second foreign key in the chain has null=True.
    def test_inheritance_null_FK(self):
        Event.objects.create()
        ScreeningNullFK.objects.create(movie=None)
        ScreeningNullFK.objects.create(movie=self.movie)

        self.assertEqual(len(Event.objects.all()), 3)
        self.assertEqual(len(Event.objects.select_related("screeningnullfk")), 3)
        self.assertEqual(len(Event.objects.select_related("screeningnullfk__movie")), 3)

        self.assertEqual(len(Event.objects.values()), 3)
        self.assertEqual(len(Event.objects.values("screeningnullfk__pk")), 3)
        self.assertEqual(len(Event.objects.values("screeningnullfk__movie__pk")), 3)
        self.assertEqual(len(Event.objects.values("screeningnullfk__movie__title")), 3)
        self.assertEqual(
            len(
                Event.objects.values(
                    "screeningnullfk__movie__pk", "screeningnullfk__movie__title"
                )
            ),
            3,
        )

        self.assertEqual(
            Event.objects.filter(screeningnullfk__movie=self.movie).count(), 1
        )
        self.assertEqual(
            Event.objects.exclude(screeningnullfk__movie=self.movie).count(), 2
        )

    def test_null_exclude(self):
        screening = ScreeningNullFK.objects.create(movie=None)
        ScreeningNullFK.objects.create(movie=self.movie)
        self.assertEqual(
            list(ScreeningNullFK.objects.exclude(movie__id=self.movie.pk)), [screening]
        )

    # This test failed in #16715 because in some cases INNER JOIN was selected
    # for the second foreign key relation instead of LEFT OUTER JOIN.
    def test_explicit_ForeignKey(self):
        Package.objects.create()
        screening = Screening.objects.create(movie=self.movie)
        Package.objects.create(screening=screening)

        self.assertEqual(len(Package.objects.all()), 2)
        self.assertEqual(len(Package.objects.select_related("screening")), 2)
        self.assertEqual(len(Package.objects.select_related("screening__movie")), 2)

        self.assertEqual(len(Package.objects.values()), 2)
        self.assertEqual(len(Package.objects.values("screening__pk")), 2)
        self.assertEqual(len(Package.objects.values("screening__movie__pk")), 2)
        self.assertEqual(len(Package.objects.values("screening__movie__title")), 2)
        # This failed.
        self.assertEqual(
            len(
                Package.objects.values(
                    "screening__movie__pk", "screening__movie__title"
                )
            ),
            2,
        )

        self.assertEqual(Package.objects.filter(screening__movie=self.movie).count(), 1)
        self.assertEqual(
            Package.objects.exclude(screening__movie=self.movie).count(), 1
        )

    # These all work because the second foreign key in the chain has null=True.
    def test_explicit_ForeignKey_NullFK(self):
        PackageNullFK.objects.create()
        screening = ScreeningNullFK.objects.create(movie=None)
        screening_with_movie = ScreeningNullFK.objects.create(movie=self.movie)
        PackageNullFK.objects.create(screening=screening)
        PackageNullFK.objects.create(screening=screening_with_movie)

        self.assertEqual(len(PackageNullFK.objects.all()), 3)
        self.assertEqual(len(PackageNullFK.objects.select_related("screening")), 3)
        self.assertEqual(
            len(PackageNullFK.objects.select_related("screening__movie")), 3
        )

        self.assertEqual(len(PackageNullFK.objects.values()), 3)
        self.assertEqual(len(PackageNullFK.objects.values("screening__pk")), 3)
        self.assertEqual(len(PackageNullFK.objects.values("screening__movie__pk")), 3)
        self.assertEqual(
            len(PackageNullFK.objects.values("screening__movie__title")), 3
        )
        self.assertEqual(
            len(
                PackageNullFK.objects.values(
                    "screening__movie__pk", "screening__movie__title"
                )
            ),
            3,
        )

        self.assertEqual(
            PackageNullFK.objects.filter(screening__movie=self.movie).count(), 1
        )
        self.assertEqual(
            PackageNullFK.objects.exclude(screening__movie=self.movie).count(), 2
        )


# Some additional tests for #16715. The only difference is the depth of the
# nesting as we now use 4 models instead of 3 (and thus 3 relations). This
# checks if promotion of join types works for deeper nesting too.
class DeeplyNestedForeignKeysTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.director = Person.objects.create(name="Terry Gilliam / Terry Jones")
        cls.movie = Movie.objects.create(
            title="Monty Python and the Holy Grail", director=cls.director
        )

    def test_inheritance(self):
        Event.objects.create()
        Screening.objects.create(movie=self.movie)

        self.assertEqual(len(Event.objects.all()), 2)
        self.assertEqual(
            len(Event.objects.select_related("screening__movie__director")), 2
        )

        self.assertEqual(len(Event.objects.values()), 2)
        self.assertEqual(len(Event.objects.values("screening__movie__director__pk")), 2)
        self.assertEqual(
            len(Event.objects.values("screening__movie__director__name")), 2
        )
        self.assertEqual(
            len(
                Event.objects.values(
                    "screening__movie__director__pk", "screening__movie__director__name"
                )
            ),
            2,
        )
        self.assertEqual(
            len(
                Event.objects.values(
                    "screening__movie__pk", "screening__movie__director__pk"
                )
            ),
            2,
        )
        self.assertEqual(
            len(
                Event.objects.values(
                    "screening__movie__pk", "screening__movie__director__name"
                )
            ),
            2,
        )
        self.assertEqual(
            len(
                Event.objects.values(
                    "screening__movie__title", "screening__movie__director__pk"
                )
            ),
            2,
        )
        self.assertEqual(
            len(
                Event.objects.values(
                    "screening__movie__title", "screening__movie__director__name"
                )
            ),
            2,
        )

        self.assertEqual(
            Event.objects.filter(screening__movie__director=self.director).count(), 1
        )
        self.assertEqual(
            Event.objects.exclude(screening__movie__director=self.director).count(), 1
        )

    def test_explicit_ForeignKey(self):
        Package.objects.create()
        screening = Screening.objects.create(movie=self.movie)
        Package.objects.create(screening=screening)

        self.assertEqual(len(Package.objects.all()), 2)
        self.assertEqual(
            len(Package.objects.select_related("screening__movie__director")), 2
        )

        self.assertEqual(len(Package.objects.values()), 2)
        self.assertEqual(
            len(Package.objects.values("screening__movie__director__pk")), 2
        )
        self.assertEqual(
            len(Package.objects.values("screening__movie__director__name")), 2
        )
        self.assertEqual(
            len(
                Package.objects.values(
                    "screening__movie__director__pk", "screening__movie__director__name"
                )
            ),
            2,
        )
        self.assertEqual(
            len(
                Package.objects.values(
                    "screening__movie__pk", "screening__movie__director__pk"
                )
            ),
            2,
        )
        self.assertEqual(
            len(
                Package.objects.values(
                    "screening__movie__pk", "screening__movie__director__name"
                )
            ),
            2,
        )
        self.assertEqual(
            len(
                Package.objects.values(
                    "screening__movie__title", "screening__movie__director__pk"
                )
            ),
            2,
        )
        self.assertEqual(
            len(
                Package.objects.values(
                    "screening__movie__title", "screening__movie__director__name"
                )
            ),
            2,
        )

        self.assertEqual(
            Package.objects.filter(screening__movie__director=self.director).count(), 1
        )
        self.assertEqual(
            Package.objects.exclude(screening__movie__director=self.director).count(), 1
        )



import os
import re
import sqlite3
import tempfile
import threading
import unittest
from contextlib import contextmanager
from pathlib import Path
from unittest import mock

from django.core.exceptions import ImproperlyConfigured
from django.db import (
    DEFAULT_DB_ALIAS,
    NotSupportedError,
    connection,
    connections,
    transaction,
)
from django.db.models import Aggregate, Avg, StdDev, Sum, Variance
from django.db.utils import ConnectionHandler
from django.test import SimpleTestCase, TestCase, TransactionTestCase, override_settings
from django.test.utils import CaptureQueriesContext, isolate_apps

from ..models import Item, Object, Square


@unittest.skipUnless(connection.vendor == "sqlite", "SQLite tests")
class Tests(TestCase):
    longMessage = True

    def test_aggregation(self):
        """Raise NotSupportedError when aggregating on date/time fields."""
        for aggregate in (Sum, Avg, Variance, StdDev):
            msg = (
                f"SQLite does not support {aggregate.__name__} on date or "
                "time fields, because they are stored as text."
            )
            with self.assertRaisesMessage(NotSupportedError, msg):
                Item.objects.aggregate(aggregate("time"))
            with self.assertRaisesMessage(NotSupportedError, msg):
                Item.objects.aggregate(aggregate("date"))
            with self.assertRaisesMessage(NotSupportedError, msg):
                Item.objects.aggregate(aggregate("last_modified"))
            with self.assertRaisesMessage(NotSupportedError, msg):
                Item.objects.aggregate(
                    **{
                        "complex": aggregate("last_modified")
                        + aggregate("last_modified")
                    }
                )

    def test_distinct_aggregation(self):
        class DistinctAggregate(Aggregate):
            allow_distinct = True

        aggregate = DistinctAggregate("first", "second", distinct=True)
        msg = (
            "SQLite doesn't support DISTINCT on aggregate functions accepting "
            "multiple arguments."
        )
        with self.assertRaisesMessage(NotSupportedError, msg):
            connection.ops.check_expression_support(aggregate)

    def test_distinct_aggregation_multiple_args_no_distinct(self):
        # Aggregate functions accept multiple arguments when DISTINCT isn't
        # used, e.g. GROUP_CONCAT().
        class DistinctAggregate(Aggregate):
            allow_distinct = True

        aggregate = DistinctAggregate("first", "second", distinct=False)
        connection.ops.check_expression_support(aggregate)

    def test_memory_db_test_name(self):
        """A named in-memory db should be allowed where supported."""
        from django.db.backends.sqlite3.base import DatabaseWrapper

        settings_dict = {
            "TEST": {
                "NAME": "file:memorydb_test?mode=memory&cache=shared",
            }
        }
        creation = DatabaseWrapper(settings_dict).creation
        self.assertEqual(
            creation._get_test_db_name(),
            creation.connection.settings_dict["TEST"]["NAME"],
        )

    def test_regexp_function(self):
        tests = (
            ("test", r"[0-9]+", False),
            ("test", r"[a-z]+", True),
            ("test", None, None),
            (None, r"[a-z]+", None),
            (None, None, None),
        )
        for string, pattern, expected in tests:
            with self.subTest((string, pattern)):
                with connection.cursor() as cursor:
                    cursor.execute("SELECT %s REGEXP %s", [string, pattern])
                    value = cursor.fetchone()[0]
                value = bool(value) if value in {0, 1} else value
                self.assertIs(value, expected)

    def test_pathlib_name(self):
        with tempfile.TemporaryDirectory() as tmp:
            settings_dict = {
                "default": {
                    "ENGINE": "django.db.backends.sqlite3",
                    "NAME": Path(tmp) / "test.db",
                },
            }
            connections = ConnectionHandler(settings_dict)
            connections["default"].ensure_connection()
            connections["default"].close()
            self.assertTrue(os.path.isfile(os.path.join(tmp, "test.db")))

    @mock.patch.object(connection, "get_database_version", return_value=(3, 36))
    def test_check_database_version_supported(self, mocked_get_database_version):
        msg = "SQLite 3.37 or later is required (found 3.36)."
        with self.assertRaisesMessage(NotSupportedError, msg):
            connection.check_database_version_supported()
        self.assertTrue(mocked_get_database_version.called)

    def test_init_command(self):
        settings_dict = {
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
                "NAME": ":memory:",
                "OPTIONS": {
                    "init_command": "PRAGMA synchronous=3; PRAGMA cache_size=2000;",
                },
            }
        }
        connections = ConnectionHandler(settings_dict)
        connections["default"].ensure_connection()
        try:
            with connections["default"].cursor() as cursor:
                cursor.execute("PRAGMA synchronous")
                value = cursor.fetchone()[0]
                self.assertEqual(value, 3)
                cursor.execute("PRAGMA cache_size")
                value = cursor.fetchone()[0]
                self.assertEqual(value, 2000)
        finally:
            connections["default"]._close()


@unittest.skipUnless(connection.vendor == "sqlite", "SQLite tests")
@isolate_apps("backends")
class SchemaTests(TransactionTestCase):
    available_apps = ["backends"]

    def test_autoincrement(self):
        """
        auto_increment fields are created with the AUTOINCREMENT keyword
        in order to be monotonically increasing (#10164).
        """
        with connection.schema_editor(collect_sql=True) as editor:
            editor.create_model(Square)
            statements = editor.collected_sql
        match = re.search('"id" ([^,]+),', statements[0])
        self.assertIsNotNone(match)
        self.assertEqual(
            "integer NOT NULL PRIMARY KEY AUTOINCREMENT",
            match[1],
            "Wrong SQL used to create an auto-increment column on SQLite",
        )

    def test_disable_constraint_checking_failure_disallowed(self):
        """
        SQLite schema editor is not usable within an outer transaction if
        foreign key constraint checks are not disabled beforehand.
        """
        msg = (
            "SQLite schema editor cannot be used while foreign key "
            "constraint checks are enabled. Make sure to disable them "
            "before entering a transaction.atomic() context because "
            "SQLite does not support disabling them in the middle of "
            "a multi-statement transaction."
        )
        with self.assertRaisesMessage(NotSupportedError, msg):
            with transaction.atomic(), connection.schema_editor(atomic=True):
                pass

    def test_constraint_checks_disabled_atomic_allowed(self):
        """
        SQLite schema editor is usable within an outer transaction as long as
        foreign key constraints checks are disabled beforehand.
        """

        def constraint_checks_enabled():
            with connection.cursor() as cursor:
                return bool(cursor.execute("PRAGMA foreign_keys").fetchone()[0])

        with connection.constraint_checks_disabled(), transaction.atomic():
            with connection.schema_editor(atomic=True):
                self.assertFalse(constraint_checks_enabled())
            self.assertFalse(constraint_checks_enabled())
        self.assertTrue(constraint_checks_enabled())


@unittest.skipUnless(connection.vendor == "sqlite", "Test only for SQLite")
@override_settings(DEBUG=True)
class LastExecutedQueryTest(TestCase):
    def test_no_interpolation(self):
        # This shouldn't raise an exception (#17158)
        query = "SELECT strftime('%Y', 'now');"
        with connection.cursor() as cursor:
            cursor.execute(query)
        self.assertEqual(connection.queries[-1]["sql"], query)

    def test_parameter_quoting(self):
        # The implementation of last_executed_queries isn't optimal. It's
        # worth testing that parameters are quoted (#14091).
        query = "SELECT %s"
        params = ["\"'\\"]
        with connection.cursor() as cursor:
            cursor.execute(query, params)
        # Note that the single quote is repeated
        substituted = "SELECT '\"''\\'"
        self.assertEqual(connection.queries[-1]["sql"], substituted)

    def test_parameter_count_exceeds_variable_or_column_limit(self):
        sql = "SELECT MAX(%s)" % ", ".join(["%s"] * 1001)
        params = list(range(1001))
        for label, limit, current_limit in [
            (
                "variable",
                sqlite3.SQLITE_LIMIT_VARIABLE_NUMBER,
                connection.features.max_query_params,
            ),
            (
                "column",
                sqlite3.SQLITE_LIMIT_COLUMN,
                connection.connection.getlimit(sqlite3.SQLITE_LIMIT_COLUMN),
            ),
        ]:
            with self.subTest(limit=label):
                connection.connection.setlimit(limit, 1000)
                self.addCleanup(connection.connection.setlimit, limit, current_limit)
                with connection.cursor() as cursor:
                    # This should not raise an exception.
                    cursor.db.ops.last_executed_query(cursor.cursor, sql, params)
                connection.connection.setlimit(limit, current_limit)


@unittest.skipUnless(connection.vendor == "sqlite", "SQLite tests")
class EscapingChecks(TestCase):
    """
    All tests in this test case are also run with settings.DEBUG=True in
    EscapingChecksDebug test case, to also test CursorDebugWrapper.
    """

    def test_parameter_escaping(self):
        # '%s' escaping support for sqlite3 (#13648).
        with connection.cursor() as cursor:
            cursor.execute("select strftime('%s', date('now'))")
            response = cursor.fetchall()[0][0]
        # response should be an non-zero integer
        self.assertTrue(int(response))


@unittest.skipUnless(connection.vendor == "sqlite", "SQLite tests")
@override_settings(DEBUG=True)
class EscapingChecksDebug(EscapingChecks):
    pass


@unittest.skipUnless(connection.vendor == "sqlite", "SQLite tests")
class ThreadSharing(TransactionTestCase):
    available_apps = ["backends"]

    def test_database_sharing_in_threads(self):
        thread_connections = []

        def create_object():
            Object.objects.create()
            thread_connections.append(connections[DEFAULT_DB_ALIAS].connection)

        main_connection = connections[DEFAULT_DB_ALIAS].connection
        try:
            create_object()
            thread = threading.Thread(target=create_object)
            thread.start()
            thread.join()
            self.assertEqual(Object.objects.count(), 2)
        finally:
            for conn in thread_connections:
                if conn is not main_connection:
                    conn.close()


@unittest.skipUnless(connection.vendor == "sqlite", "SQLite tests")
class TestTransactionMode(SimpleTestCase):
    databases = {"default"}

    def test_default_transaction_mode(self):
        with CaptureQueriesContext(connection) as captured_queries:
            with transaction.atomic():
                pass

        begin_query, commit_query = captured_queries
        self.assertEqual(begin_query["sql"], "BEGIN")
        self.assertEqual(commit_query["sql"], "COMMIT")

    def test_invalid_transaction_mode(self):
        msg = (
            "settings.DATABASES['default']['OPTIONS']['transaction_mode'] is "
            "improperly configured to 'invalid'. Use one of 'DEFERRED', 'EXCLUSIVE', "
            "'IMMEDIATE', or None."
        )
        with self.change_transaction_mode("invalid") as new_connection:
            with self.assertRaisesMessage(ImproperlyConfigured, msg):
                new_connection.ensure_connection()

    def test_valid_transaction_modes(self):
        valid_transaction_modes = ("deferred", "immediate", "exclusive")
        for transaction_mode in valid_transaction_modes:
            with (
                self.subTest(transaction_mode=transaction_mode),
                self.change_transaction_mode(transaction_mode) as new_connection,
                CaptureQueriesContext(new_connection) as captured_queries,
            ):
                new_connection.set_autocommit(
                    False, force_begin_transaction_with_broken_autocommit=True
                )
                new_connection.commit()
                expected_transaction_mode = transaction_mode.upper()
                begin_sql = captured_queries[0]["sql"]
                self.assertEqual(begin_sql, f"BEGIN {expected_transaction_mode}")

    @contextmanager
    def change_transaction_mode(self, transaction_mode):
        new_connection = connection.copy()
        new_connection.settings_dict["OPTIONS"] = {
            **new_connection.settings_dict["OPTIONS"],
            "transaction_mode": transaction_mode,
        }
        try:
            yield new_connection
        finally:
            new_connection._close()



from django.core.exceptions import FieldFetchBlocked


class FetchMode:
    __slots__ = ()

    track_peers = False

    def fetch(self, fetcher, instance):
        raise NotImplementedError("Subclasses must implement this method.")


class FetchOne(FetchMode):
    __slots__ = ()

    def fetch(self, fetcher, instance):
        fetcher.fetch_one(instance)

    def __reduce__(self):
        return "FETCH_ONE"


FETCH_ONE = FetchOne()


class FetchPeers(FetchMode):
    __slots__ = ()

    track_peers = True

    def fetch(self, fetcher, instance):
        instances = [
            peer
            for peer_weakref in instance._state.peers
            if (peer := peer_weakref()) is not None
        ]
        if len(instances) > 1:
            fetcher.fetch_many(instances)
        else:
            fetcher.fetch_one(instance)

    def __reduce__(self):
        return "FETCH_PEERS"


FETCH_PEERS = FetchPeers()


class FetchRaise(FetchMode):
    __slots__ = ()

    def fetch(self, fetcher, instance):
        klass = instance.__class__.__qualname__
        field_name = fetcher.field.name
        raise FieldFetchBlocked(f"Fetching of {klass}.{field_name} blocked.") from None

    def __reduce__(self):
        return "FETCH_RAISE"


FETCH_RAISE = FetchRaise()



from django import forms
from django.contrib.admin.forms import AdminAuthenticationForm, AdminPasswordChangeForm
from django.contrib.admin.helpers import ActionForm
from django.core.exceptions import ValidationError

from .models import Section


class CustomAdminAuthenticationForm(AdminAuthenticationForm):
    class Media:
        css = {"all": ("path/to/media.css",)}

    def clean_username(self):
        username = self.cleaned_data.get("username")
        if username == "customform":
            raise ValidationError("custom form error")
        return username


class CustomAdminPasswordChangeForm(AdminPasswordChangeForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["old_password"].label = "Custom old password label"


class MediaActionForm(ActionForm):
    class Media:
        js = ["path/to/media.js"]


class SectionFormWithOptgroups(forms.ModelForm):
    articles = forms.ChoiceField(
        choices=[
            ("Published", [("1", "Test Article")]),
            ("Draft", [("2", "Other Article")]),
        ],
        required=False,
    )

    class Meta:
        model = Section
        fields = ["name", "articles"]


class SectionFormWithObjectOptgroups(forms.ModelForm):
    """Form with model instances as optgroup keys (tests str() conversion)."""

    articles = forms.ChoiceField(required=False)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Use Section instances as optgroup keys
        sections = Section.objects.all()[:2]
        if sections:
            self.fields["articles"].choices = [
                (sections[0], [("1", "Article 1")]),
                (
                    sections[1] if len(sections) > 1 else sections[0],
                    [("2", "Article 2")],
                ),
            ]

    class Meta:
        model = Section
        fields = ["name", "articles"]


class SectionFormWithDynamicOptgroups(forms.ModelForm):
    """
    Form where the field with optgroups is added dynamically in __init__.
    This tests that the implementation doesn't rely on accessing the
    uninstantiated form class's _meta or fields, which wouldn't work here.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Dynamically add a field with optgroups after instantiation.
        self.fields["articles"] = forms.ChoiceField(
            choices=[
                ("Category A", [("1", "Item 1"), ("2", "Item 2")]),
                ("Category B", [("3", "Item 3"), ("4", "Item 4")]),
            ],
            required=False,
        )

    class Meta:
        model = Section
        fields = ["name"]



from django.conf import settings
from django.core.mail import InvalidMailer, MailerDoesNotExist
from django.utils.module_loading import import_string

DEFAULT_MAILER_ALIAS = "default"

# Default value for a MAILERS "BACKEND". (Not related to the default mailer.)
DEFAULT_MAILER_BACKEND = "django.core.mail.backends.smtp.EmailBackend"


class MailersHandler:
    def __getitem__(self, /, alias):
        return self.create_connection(alias)

    def __contains__(self, /, alias):
        return alias in self.settings

    def __iter__(self):
        return iter(self.settings)

    def get(self, alias, /, default=None):
        try:
            return self[alias]
        except MailerDoesNotExist:
            return default

    @property
    def default(self):
        return self[DEFAULT_MAILER_ALIAS]

    @property
    def settings(self):
        # RemovedInDjango70Warning: change to:
        #   return settings.MAILERS
        return getattr(settings, "MAILERS", {})

    # RemovedInDjango70Warning.
    @property
    def _is_configured(self):
        """True if settings.py has opted into MAILERS support."""
        return hasattr(settings, "MAILERS")

    # RemovedInDjango70Warning: _deprecated_kwargs.
    def create_connection(self, alias, /, *, _deprecated_kwargs=None):
        # RemovedInDjango70Warning.
        if not self._is_configured and alias == DEFAULT_MAILER_ALIAS:
            # Create mailers.default from deprecated settings.
            from django.core.mail import get_connection

            assert _deprecated_kwargs is None
            return get_connection()

        try:
            config = self.settings[alias]
        except KeyError:
            raise MailerDoesNotExist(alias=alias) from None

        options = config.get("OPTIONS", {})
        if "alias" in options:
            raise InvalidMailer("OPTIONS must not define 'alias'.", alias=alias)

        # RemovedInDjango70Warning.
        if _deprecated_kwargs:
            # Being called from get_connection() to create default mailer
            # instance with some overrides. _ignore_unknown_kwargs prevents
            # BaseEmailBackend from reporting these as unknown OPTIONS.
            assert alias == DEFAULT_MAILER_ALIAS
            options = options | _deprecated_kwargs
            options |= {"_ignore_unknown_kwargs": set(_deprecated_kwargs)}

        backend_path = config.get("BACKEND", DEFAULT_MAILER_BACKEND)
        try:
            backend_class = import_string(backend_path)
        except ImportError as error:
            raise InvalidMailer(
                f"Could not find BACKEND {backend_path!r}: {error}", alias=alias
            ) from error
        return backend_class(alias=alias, **options)



"""
A Python "serializer". Doesn't do much serializing per se -- just converts to
and from basic Python data types (lists, dicts, strings, etc.). Useful as a
basis for other serializers.
"""

from django.apps import apps
from django.core.serializers import base
from django.db import DEFAULT_DB_ALIAS, models
from django.db.models import CompositePrimaryKey
from django.utils.encoding import is_protected_type


class Serializer(base.Serializer):
    """
    Serialize a QuerySet to basic Python objects.
    """

    internal_use_only = True

    def start_serialization(self):
        self._current = None
        self.objects = []

    def end_serialization(self):
        pass

    def start_object(self, obj):
        self._current = {}

    def end_object(self, obj):
        self.objects.append(self.get_dump_object(obj))
        self._current = None

    def get_dump_object(self, obj):
        data = {"model": str(obj._meta)}
        if not self.use_natural_primary_keys or not self._resolve_natural_key(obj):
            data["pk"] = self._value_from_field(obj, obj._meta.pk)
        data["fields"] = self._current
        return data

    def _value_from_field(self, obj, field):
        if isinstance(field, CompositePrimaryKey):
            return [self._value_from_field(obj, f) for f in field]
        value = field.value_from_object(obj)
        # Protected types (i.e., primitives like None, numbers, dates,
        # and Decimals) are passed through as is. All other values are
        # converted to string first.
        return value if is_protected_type(value) else field.value_to_string(obj)

    def handle_field(self, obj, field):
        self._current[field.name] = self._value_from_field(obj, field)

    def handle_fk_field(self, obj, field):
        if self.use_natural_foreign_keys and (
            natural_key_value := self._resolve_fk_natural_key(obj, field)
        ):
            value = natural_key_value
        else:
            value = self._value_from_field(obj, field)
        self._current[field.name] = value

    def handle_m2m_field(self, obj, field):
        if field.remote_field.through._meta.auto_created:
            if self.use_natural_foreign_keys and self._model_supports_natural_key(
                field.remote_field.model
            ):

                def m2m_value(value):
                    if natural := value.natural_key():
                        return natural
                    else:
                        return self._value_from_field(value, value._meta.pk)

                def queryset_iterator(obj, field):
                    attr = getattr(obj, field.name)
                    chunk_size = (
                        2000 if getattr(attr, "prefetch_cache_name", None) else None
                    )
                    query_set = attr.all()
                    if not query_set.totally_ordered:
                        current_ordering = (
                            query_set.query.order_by
                            or query_set.model._meta.ordering
                            or []
                        )
                        query_set = query_set.order_by(*current_ordering, "pk")
                    return query_set.iterator(chunk_size)

            else:

                def m2m_value(value):
                    return self._value_from_field(value, value._meta.pk)

                def queryset_iterator(obj, field):
                    query_set = getattr(obj, field.name).select_related(None).only("pk")
                    if not query_set.totally_ordered:
                        current_ordering = (
                            query_set.query.order_by
                            or query_set.model._meta.ordering
                            or []
                        )
                        query_set = query_set.order_by(*current_ordering, "pk")
                    chunk_size = 2000 if query_set._prefetch_related_lookups else None
                    return query_set.iterator(chunk_size=chunk_size)

            m2m_iter = getattr(obj, "_prefetched_objects_cache", {}).get(
                field.name,
                queryset_iterator(obj, field),
            )
            self._current[field.name] = [m2m_value(related) for related in m2m_iter]

    def getvalue(self):
        return self.objects


class Deserializer(base.Deserializer):
    """
    Deserialize simple Python objects back into Django ORM instances.

    It's expected that you pass the Python objects themselves (instead of a
    stream or a string) to the constructor
    """

    def __init__(
        self, object_list, *, using=DEFAULT_DB_ALIAS, ignorenonexistent=False, **options
    ):
        super().__init__(object_list, **options)
        self.handle_forward_references = options.pop("handle_forward_references", False)
        self.using = using
        self.ignorenonexistent = ignorenonexistent
        self.field_names_cache = {}  # Model: <list of field_names>
        self._iterator = None

    def __iter__(self):
        for obj in self.stream:
            yield from self._handle_object(obj)

    def __next__(self):
        if self._iterator is None:
            self._iterator = iter(self)
        return next(self._iterator)

    def _handle_object(self, obj):
        data = {}
        m2m_data = {}
        deferred_fields = {}

        # Look up the model and starting build a dict of data for it.
        try:
            Model = self._get_model_from_node(obj["model"])
        except base.DeserializationError:
            if self.ignorenonexistent:
                return
            raise
        if "pk" in obj:
            try:
                data[Model._meta.pk.attname] = Model._meta.pk.to_python(obj.get("pk"))
            except Exception as e:
                raise base.DeserializationError.WithData(
                    e, obj["model"], obj.get("pk"), None
                )

        if Model not in self.field_names_cache:
            self.field_names_cache[Model] = {f.name for f in Model._meta.get_fields()}
        field_names = self.field_names_cache[Model]

        # Handle each field
        for field_name, field_value in obj["fields"].items():
            if self.ignorenonexistent and field_name not in field_names:
                # skip fields no longer on model
                continue

            field = Model._meta.get_field(field_name)

            # Handle M2M relations
            if field.remote_field and isinstance(
                field.remote_field, models.ManyToManyRel
            ):
                try:
                    values = self._handle_m2m_field_node(field, field_value)
                    if values == base.DEFER_FIELD:
                        deferred_fields[field] = field_value
                    else:
                        m2m_data[field.name] = values
                except base.M2MDeserializationError as e:
                    raise base.DeserializationError.WithData(
                        e.original_exc, obj["model"], obj.get("pk"), e.pk
                    )

            # Handle FK fields
            elif field.remote_field and isinstance(
                field.remote_field, models.ManyToOneRel
            ):
                try:
                    value = self._handle_fk_field_node(field, field_value)
                    if value == base.DEFER_FIELD:
                        deferred_fields[field] = field_value
                    else:
                        data[field.attname] = value
                except Exception as e:
                    raise base.DeserializationError.WithData(
                        e, obj["model"], obj.get("pk"), field_value
                    )

            # Handle all other fields
            else:
                try:
                    data[field.name] = field.to_python(field_value)
                except Exception as e:
                    raise base.DeserializationError.WithData(
                        e, obj["model"], obj.get("pk"), field_value
                    )

        model_instance = base.build_instance(Model, data, self.using)
        yield base.DeserializedObject(model_instance, m2m_data, deferred_fields)

    def _handle_m2m_field_node(self, field, field_value):
        return base.deserialize_m2m_values(
            field, field_value, self.using, self.handle_forward_references
        )

    def _handle_fk_field_node(self, field, field_value):
        return base.deserialize_fk_value(
            field, field_value, self.using, self.handle_forward_references
        )

    @staticmethod
    def _get_model_from_node(model_identifier):
        """Look up a model from an "app_label.model_name" string."""
        try:
            return apps.get_model(model_identifier)
        except (LookupError, TypeError):
            raise base.DeserializationError(
                f"Invalid model identifier: {model_identifier}"
            )



import pickle
import warnings

from django import forms
from django.core.exceptions import ValidationError
from django.db import connection, models
from django.test import SimpleTestCase, TestCase
from django.utils.choices import CallableChoiceIterator
from django.utils.deprecation import RemovedInDjango70Warning
from django.utils.functional import lazy

from .models import (
    Bar,
    Choiceful,
    Foo,
    RenamedField,
    VerboseNameField,
    Whiz,
    WhizDelayed,
    WhizIter,
    WhizIterEmpty,
)


class Nested:
    class Field(models.Field):
        pass


class BasicFieldTests(SimpleTestCase):
    def test_show_hidden_initial(self):
        """
        Fields with choices respect show_hidden_initial as a kwarg to
        formfield().
        """
        choices = [(0, 0), (1, 1)]
        model_field = models.Field(choices=choices)
        form_field = model_field.formfield(show_hidden_initial=True)
        self.assertTrue(form_field.show_hidden_initial)

        form_field = model_field.formfield(show_hidden_initial=False)
        self.assertFalse(form_field.show_hidden_initial)

    def test_field_repr(self):
        """
        __repr__() of a field displays its name.
        """
        f = Foo._meta.get_field("a")
        self.assertEqual(repr(f), "<django.db.models.fields.CharField: a>")
        f = models.fields.CharField()
        self.assertEqual(repr(f), "<django.db.models.fields.CharField>")

    def test_field_repr_nested(self):
        """__repr__() uses __qualname__ for nested class support."""
        self.assertEqual(repr(Nested.Field()), "<model_fields.tests.Nested.Field>")

    def test_field_name(self):
        """
        A defined field name (name="fieldname") is used instead of the model
        model's attribute name (modelname).
        """
        instance = RenamedField()
        self.assertTrue(hasattr(instance, "get_fieldname_display"))
        self.assertFalse(hasattr(instance, "get_modelname_display"))

    def test_field_verbose_name(self):
        m = VerboseNameField
        for i in range(1, 22):
            self.assertEqual(
                m._meta.get_field("field%d" % i).verbose_name, "verbose field%d" % i
            )

        self.assertEqual(m._meta.get_field("id").verbose_name, "verbose pk")

    def test_choices_form_class(self):
        """Can supply a custom choices form class to Field.formfield()"""
        choices = [("a", "a")]
        field = models.CharField(choices=choices)
        klass = forms.TypedMultipleChoiceField
        self.assertIsInstance(field.formfield(choices_form_class=klass), klass)

    def test_formfield_disabled(self):
        """Field.formfield() sets disabled for fields with choices."""
        field = models.CharField(choices=[("a", "b")])
        form_field = field.formfield(disabled=True)
        self.assertIs(form_field.disabled, True)

    def test_field_str(self):
        f = models.Field()
        self.assertEqual(str(f), "<django.db.models.fields.Field>")
        f = Foo._meta.get_field("a")
        self.assertEqual(str(f), "model_fields.Foo.a")

    def test_field_ordering(self):
        """Fields are ordered based on their creation."""
        f1 = models.Field()
        f2 = models.Field(auto_created=True)
        f3 = models.Field()
        self.assertLess(f2, f1)
        self.assertGreater(f3, f1)
        self.assertIsNotNone(f1)
        self.assertNotIn(f2, (None, 1, ""))

    def test_field_instance_is_picklable(self):
        """Field instances can be pickled."""
        field = models.Field(max_length=100, default="a string")
        # Must be picklable with this cached property populated (#28188).
        field._get_default
        pickle.dumps(field)

    def test_deconstruct_nested_field(self):
        """deconstruct() uses __qualname__ for nested class support."""
        name, path, args, kwargs = Nested.Field().deconstruct()
        self.assertEqual(path, "model_fields.tests.Nested.Field")

    def test_abstract_inherited_fields(self):
        """Field instances from abstract models are not equal."""

        class AbstractModel(models.Model):
            field = models.IntegerField()

            class Meta:
                abstract = True

        class InheritAbstractModel1(AbstractModel):
            pass

        class InheritAbstractModel2(AbstractModel):
            pass

        abstract_model_field = AbstractModel._meta.get_field("field")
        inherit1_model_field = InheritAbstractModel1._meta.get_field("field")
        inherit2_model_field = InheritAbstractModel2._meta.get_field("field")

        self.assertNotEqual(abstract_model_field, inherit1_model_field)
        self.assertNotEqual(abstract_model_field, inherit2_model_field)
        self.assertNotEqual(inherit1_model_field, inherit2_model_field)

        self.assertLess(abstract_model_field, inherit1_model_field)
        self.assertLess(abstract_model_field, inherit2_model_field)
        self.assertLess(inherit1_model_field, inherit2_model_field)

    def test_hash_immutability(self):
        field = models.IntegerField()
        field_hash = hash(field)

        class MyModel(models.Model):
            rank = field

        self.assertEqual(field_hash, hash(field))

    def test_get_placeholder_deprecation(self):
        # RemovedInDjango70Warning: When the deprecation ends, remove
        # completely.
        msg = (
            "Field.get_placeholder is deprecated in favor of get_placeholder_sql. "
            "Define model_fields.tests.BasicFieldTests."
            "test_get_placeholder_deprecation.<locals>.SomeField.get_placeholder_sql "
            "to return both SQL and parameters instead."
        )
        with self.assertWarnsMessage(RemovedInDjango70Warning, msg):

            class SomeField(models.Field):
                def get_placeholder(self, value, compiler, connection):
                    return "%s"

    def test_get_placeholder_sql_shim(self):
        # RemovedInDjango70Warning: When the deprecation ends, remove
        # completely.
        with warnings.catch_warnings(record=True):
            warnings.simplefilter("always")

            class SomeField(models.Field):
                def get_placeholder(self, value, compiler, connection):
                    return "(1 + %s)"

        compiler = Bar.objects.all().query.get_compiler(connection=connection)
        self.assertEqual(
            SomeField().get_placeholder_sql(2, compiler, connection),
            ("(1 + %s)", (2,)),
        )
        self.assertEqual(
            SomeField().get_placeholder_sql(models.Value(2), compiler, connection),
            ("(1 + %s)", (2,)),
        )


class ChoicesTests(SimpleTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.no_choices = Choiceful._meta.get_field("no_choices")
        cls.empty_choices = Choiceful._meta.get_field("empty_choices")
        cls.empty_choices_bool = Choiceful._meta.get_field("empty_choices_bool")
        cls.empty_choices_text = Choiceful._meta.get_field("empty_choices_text")
        cls.with_choices = Choiceful._meta.get_field("with_choices")
        cls.with_choices_dict = Choiceful._meta.get_field("with_choices_dict")
        cls.with_choices_nested_dict = Choiceful._meta.get_field(
            "with_choices_nested_dict"
        )
        cls.choices_from_enum = Choiceful._meta.get_field("choices_from_enum")
        cls.choices_from_iterator = Choiceful._meta.get_field("choices_from_iterator")
        cls.choices_from_callable = Choiceful._meta.get_field("choices_from_callable")

    def test_choices(self):
        self.assertIsNone(self.no_choices.choices)
        self.assertEqual(self.empty_choices.choices, [])
        self.assertEqual(self.empty_choices_bool.choices, [])
        self.assertEqual(self.empty_choices_text.choices, [])
        self.assertEqual(self.with_choices.choices, [(1, "A")])
        self.assertEqual(self.with_choices_dict.choices, [(1, "A")])
        self.assertEqual(self.with_choices_nested_dict.choices, [("Thing", [(1, "A")])])
        self.assertEqual(
            self.choices_from_iterator.choices, [(0, "0"), (1, "1"), (2, "2")]
        )
        self.assertIsInstance(
            self.choices_from_callable.choices, CallableChoiceIterator
        )
        self.assertEqual(
            self.choices_from_callable.choices.func(), [(0, "0"), (1, "1"), (2, "2")]
        )

    def test_choices_slice(self):
        for choices, expected_slice in [
            (self.empty_choices.choices, []),
            (self.empty_choices_bool.choices, []),
            (self.empty_choices_text.choices, []),
            (self.with_choices.choices, [(1, "A")]),
            (self.with_choices_dict.choices, [(1, "A")]),
            (self.with_choices_nested_dict.choices, [("Thing", [(1, "A")])]),
            (self.choices_from_iterator.choices, [(0, "0"), (1, "1")]),
            (self.choices_from_callable.choices.func(), [(0, "0"), (1, "1")]),
            (self.choices_from_callable.choices, [(0, "0"), (1, "1")]),
        ]:
            with self.subTest(choices=choices):
                self.assertEqual(choices[:2], expected_slice)

    def test_choices_negative_index(self):
        for choices, expected_choice in [
            (self.with_choices.choices, (1, "A")),
            (self.with_choices_dict.choices, (1, "A")),
            (self.with_choices_nested_dict.choices, ("Thing", [(1, "A")])),
            (self.choices_from_iterator.choices, (2, "2")),
            (self.choices_from_callable.choices.func(), (2, "2")),
            (self.choices_from_callable.choices, (2, "2")),
        ]:
            with self.subTest(choices=choices):
                self.assertEqual(choices[-1], expected_choice)

    def test_flatchoices(self):
        self.assertEqual(self.no_choices.flatchoices, [])
        self.assertEqual(self.empty_choices.flatchoices, [])
        self.assertEqual(self.empty_choices_bool.flatchoices, [])
        self.assertEqual(self.empty_choices_text.flatchoices, [])
        self.assertEqual(self.with_choices.flatchoices, [(1, "A")])
        self.assertEqual(self.with_choices_dict.flatchoices, [(1, "A")])
        self.assertEqual(self.with_choices_nested_dict.flatchoices, [(1, "A")])
        self.assertEqual(
            self.choices_from_iterator.flatchoices, [(0, "0"), (1, "1"), (2, "2")]
        )
        self.assertEqual(
            self.choices_from_callable.flatchoices, [(0, "0"), (1, "1"), (2, "2")]
        )

    def test_check(self):
        self.assertEqual(Choiceful.check(), [])

    def test_invalid_choice(self):
        model_instance = None  # Actual model instance not needed.
        self.no_choices.validate(0, model_instance)
        msg = "['Value 99 is not a valid choice.']"
        with self.assertRaisesMessage(ValidationError, msg):
            self.empty_choices.validate(99, model_instance)
        with self.assertRaisesMessage(ValidationError, msg):
            self.with_choices.validate(99, model_instance)

    def test_formfield(self):
        no_choices_formfield = self.no_choices.formfield()
        self.assertIsInstance(no_choices_formfield, forms.IntegerField)
        fields = (
            self.empty_choices,
            self.empty_choices_bool,
            self.empty_choices_text,
            self.with_choices,
            self.with_choices_dict,
            self.with_choices_nested_dict,
            self.choices_from_enum,
            self.choices_from_iterator,
            self.choices_from_callable,
        )
        for field in fields:
            with self.subTest(field=field):
                self.assertIsInstance(field.formfield(), forms.ChoiceField)

    def test_choices_from_enum(self):
        # Choices class was transparently resolved when given as argument.
        self.assertEqual(self.choices_from_enum.choices, Choiceful.Suit.choices)
        self.assertEqual(self.choices_from_enum.flatchoices, Choiceful.Suit.choices)


class GetFieldDisplayTests(SimpleTestCase):
    def test_choices_and_field_display(self):
        """
        get_choices() interacts with get_FIELD_display() to return the expected
        values.
        """
        self.assertEqual(Whiz(c=1).get_c_display(), "First")  # A nested value
        self.assertEqual(Whiz(c=0).get_c_display(), "Other")  # A top level value
        self.assertEqual(Whiz(c=9).get_c_display(), 9)  # Invalid value
        self.assertIsNone(Whiz(c=None).get_c_display())  # Blank value
        self.assertEqual(Whiz(c="").get_c_display(), "")  # Empty value
        self.assertEqual(WhizDelayed(c=0).get_c_display(), "Other")  # Delayed choices

    def test_get_FIELD_display_translated(self):
        """A translated display value is coerced to str."""
        val = Whiz(c=5).get_c_display()
        self.assertIsInstance(val, str)
        self.assertEqual(val, "translated")

    def test_overriding_FIELD_display(self):
        class FooBar(models.Model):
            foo_bar = models.IntegerField(choices=[(1, "foo"), (2, "bar")])

            def get_foo_bar_display(self):
                return "something"

        f = FooBar(foo_bar=1)
        self.assertEqual(f.get_foo_bar_display(), "something")

    def test_overriding_inherited_FIELD_display(self):
        class Base(models.Model):
            foo = models.CharField(max_length=254, choices=[("A", "Base A")])

            class Meta:
                abstract = True

        class Child(Base):
            foo = models.CharField(
                max_length=254, choices=[("A", "Child A"), ("B", "Child B")]
            )

        self.assertEqual(Child(foo="A").get_foo_display(), "Child A")
        self.assertEqual(Child(foo="B").get_foo_display(), "Child B")

    def test_iterator_choices(self):
        """
        get_choices() works with Iterators.
        """
        self.assertEqual(WhizIter(c=1).c, 1)  # A nested value
        self.assertEqual(WhizIter(c=9).c, 9)  # Invalid value
        self.assertIsNone(WhizIter(c=None).c)  # Blank value
        self.assertEqual(WhizIter(c="").c, "")  # Empty value

    def test_empty_iterator_choices(self):
        """
        get_choices() works with empty iterators.
        """
        self.assertEqual(WhizIterEmpty(c="a").c, "a")  # A nested value
        self.assertEqual(WhizIterEmpty(c="b").c, "b")  # Invalid value
        self.assertIsNone(WhizIterEmpty(c=None).c)  # Blank value
        self.assertEqual(WhizIterEmpty(c="").c, "")  # Empty value


class GetChoicesTests(SimpleTestCase):
    def test_empty_choices(self):
        choices = []
        f = models.CharField(choices=choices)
        self.assertEqual(f.get_choices(include_blank=False), choices)

    def test_blank_in_choices(self):
        choices = [("", "<><>"), ("a", "A")]
        f = models.CharField(choices=choices)
        self.assertEqual(f.get_choices(include_blank=True), choices)

    def test_blank_in_grouped_choices(self):
        choices = [
            ("f", "Foo"),
            ("b", "Bar"),
            (
                "Group",
                [
                    ("", "No Preference"),
                    ("fg", "Foo"),
                    ("bg", "Bar"),
                ],
            ),
        ]
        f = models.CharField(choices=choices)
        self.assertEqual(f.get_choices(include_blank=True), choices)

    def test_lazy_strings_not_evaluated(self):
        lazy_func = lazy(lambda x: 0 / 0, int)  # raises ZeroDivisionError if evaluated.
        f = models.CharField(choices=[(lazy_func("group"), [("a", "A"), ("b", "B")])])
        self.assertEqual(
            f.get_choices(include_blank=True)[0],
            ("", models.utils.get_blank_choice_label()),
        )


class GetChoicesOrderingTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.foo1 = Foo.objects.create(a="a", d="12.35")
        cls.foo2 = Foo.objects.create(a="b", d="12.34")
        cls.bar1 = Bar.objects.create(a=cls.foo1, b="b")
        cls.bar2 = Bar.objects.create(a=cls.foo2, b="a")
        cls.field = Bar._meta.get_field("a")

    def assertChoicesEqual(self, choices, objs):
        self.assertEqual(choices, [(obj.pk, str(obj)) for obj in objs])

    def test_get_choices(self):
        self.assertChoicesEqual(
            self.field.get_choices(include_blank=False, ordering=("a",)),
            [self.foo1, self.foo2],
        )
        self.assertChoicesEqual(
            self.field.get_choices(include_blank=False, ordering=("-a",)),
            [self.foo2, self.foo1],
        )

    def test_get_choices_default_ordering(self):
        self.addCleanup(setattr, Foo._meta, "ordering", Foo._meta.ordering)
        Foo._meta.ordering = ("d",)
        self.assertChoicesEqual(
            self.field.get_choices(include_blank=False), [self.foo2, self.foo1]
        )

    def test_get_choices_reverse_related_field(self):
        self.assertChoicesEqual(
            self.field.remote_field.get_choices(include_blank=False, ordering=("a",)),
            [self.bar1, self.bar2],
        )
        self.assertChoicesEqual(
            self.field.remote_field.get_choices(include_blank=False, ordering=("-a",)),
            [self.bar2, self.bar1],
        )

    def test_get_choices_reverse_related_field_default_ordering(self):
        self.addCleanup(setattr, Bar._meta, "ordering", Bar._meta.ordering)
        Bar._meta.ordering = ("b",)
        self.assertChoicesEqual(
            self.field.remote_field.get_choices(include_blank=False),
            [self.bar2, self.bar1],
        )


class GetChoicesLimitChoicesToTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.foo1 = Foo.objects.create(a="a", d="12.34")
        cls.foo2 = Foo.objects.create(a="b", d="12.34")
        cls.bar1 = Bar.objects.create(a=cls.foo1, b="b")
        cls.bar2 = Bar.objects.create(a=cls.foo2, b="a")
        cls.field = Bar._meta.get_field("a")

    def assertChoicesEqual(self, choices, objs):
        self.assertCountEqual(choices, [(obj.pk, str(obj)) for obj in objs])

    def test_get_choices(self):
        self.assertChoicesEqual(
            self.field.get_choices(include_blank=False, limit_choices_to={"a": "a"}),
            [self.foo1],
        )
        self.assertChoicesEqual(
            self.field.get_choices(include_blank=False, limit_choices_to={}),
            [self.foo1, self.foo2],
        )

    def test_get_choices_reverse_related_field(self):
        field = self.field.remote_field
        self.assertChoicesEqual(
            field.get_choices(include_blank=False, limit_choices_to={"b": "b"}),
            [self.bar1],
        )
        self.assertChoicesEqual(
            field.get_choices(include_blank=False, limit_choices_to={}),
            [self.bar1, self.bar2],
        )



"""Email backend that writes messages to a file."""

import datetime
import os

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import InvalidMailer
from django.core.mail.backends.console import EmailBackend as ConsoleEmailBackend


class EmailBackend(ConsoleEmailBackend):
    def __init__(self, fail_silently=False, file_path=None, **kwargs):
        self._fname = None
        # Since we're using the console-based backend as a base, force the
        # stream to be None, so we don't default to stdout.
        kwargs["stream"] = None
        super().__init__(fail_silently=fail_silently, **kwargs)

        # RemovedInDjango70Warning.
        if self.alias is None:
            # Use deprecated settings when MAILERS not enabled.
            if file_path is not None:
                self.file_path = file_path
            else:
                self.file_path = getattr(settings, "EMAIL_FILE_PATH", None)
            if self.file_path is None:
                raise ImproperlyConfigured(
                    "The EMAIL_FILE_PATH setting must be set to use the file "
                    "EmailBackend."
                )
            self.file_path = os.path.abspath(self.file_path)
            try:
                os.makedirs(self.file_path, exist_ok=True)
            except FileExistsError:
                raise ImproperlyConfigured(
                    "Path for saving email messages exists, but is not a directory: %s"
                    % self.file_path
                )
            except OSError as err:
                raise ImproperlyConfigured(
                    "Could not create directory for saving email messages: %s (%s)"
                    % (self.file_path, err)
                )
            # Make sure that self.file_path is writable.
            if not os.access(self.file_path, os.W_OK):
                raise ImproperlyConfigured(
                    "Could not write to directory: %s" % self.file_path
                )
            return

        if file_path is None:
            raise InvalidMailer("OPTIONS must define 'file_path'.", alias=self.alias)
        self.file_path = os.path.abspath(file_path)
        try:
            os.makedirs(self.file_path, exist_ok=True)
        except FileExistsError:
            raise InvalidMailer(
                f"'file_path' is not a directory: {self.file_path}",
                alias=self.alias,
            )
        except OSError as err:
            raise InvalidMailer(
                f"Could not create 'file_path': {self.file_path} ({err})",
                alias=self.alias,
            )
        if not os.access(self.file_path, os.W_OK):
            raise InvalidMailer(
                f"'file_path' is not writable: {self.file_path}",
                alias=self.alias,
            )

    def write_message(self, message):
        self.stream.write(message.message().as_bytes() + b"\n")
        self.stream.write(b"-" * 79)
        self.stream.write(b"\n")

    def _get_filename(self):
        """Return a unique file name."""
        if self._fname is None:
            timestamp = datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
            fname = "%s-%s.log" % (timestamp, abs(id(self)))
            self._fname = os.path.join(self.file_path, fname)
        return self._fname

    def open(self):
        if self.stream is None:
            self.stream = open(self._get_filename(), "ab")
            return True
        return False

    def close(self):
        try:
            if self.stream is not None:
                self.stream.close()
        finally:
            self.stream = None



from unittest import mock

from django.db import DEFAULT_DB_ALIAS, DatabaseError, connection
from django.db.models.sql import Query
from django.db.models.sql.compiler import SQLCompiler
from django.test import TestCase
from django.utils.deprecation import RemovedInDjango70Warning

from .models import Item


class SQLCompilerTest(TestCase):
    def test_repr(self):
        query = Query(Item)
        compiler = query.get_compiler(DEFAULT_DB_ALIAS, connection)
        self.assertEqual(
            repr(compiler),
            f"<SQLCompiler model=Item connection="
            f"<DatabaseWrapper vendor={connection.vendor!r} alias='default'> "
            f"using='default'>",
        )

    def test_execute_sql_suppresses_cursor_closing_failure_on_exception(self):
        query = Query(Item)
        compiler = SQLCompiler(query, connection, None)
        cursor = mock.MagicMock()

        # When execution fails, the cursor may have been closed.
        # Django's attempt to close it again will fail, and needs catching.
        execute_err = DatabaseError("execute failed")
        cursor.execute.side_effect = execute_err
        cursor.close.side_effect = DatabaseError("close failed")

        with mock.patch.object(connection, "cursor", return_value=cursor):
            with self.assertRaises(DatabaseError) as ctx:
                compiler.execute_sql("SELECT 1", [])

        # There is no irrelevant context from trying to close a closed cursor.
        exc = ctx.exception
        self.assertIs(exc, execute_err)
        self.assertIsNone(exc.__cause__)
        self.assertTrue(exc.__suppress_context__)

    # RemovedInDjango70Warning: When the deprecation ends, remove this
    # test.
    def test_quote_name_unless_alias_deprecation(self):
        query = Query(Item)
        compiler = SQLCompiler(query, connection, None)
        msg = (
            "SQLCompiler.quote_name_unless_alias() is deprecated. "
            "Use .quote_name() instead."
        )
        with self.assertWarnsMessage(RemovedInDjango70Warning, msg) as ctx:
            self.assertEqual(
                compiler.quote_name_unless_alias("name"),
                compiler.quote_name("name"),
            )
        self.assertEqual(ctx.filename, __file__)



import math
from unittest import skipIf
from unittest.mock import patch

from django.contrib.gis.geos import GEOSGeometry, LineString
from django.contrib.gis.geos import prototypes as capi
from django.contrib.gis.geos.coordseq import GEOSCoordSeq
from django.contrib.gis.geos.libgeos import geos_version_tuple
from django.test import SimpleTestCase


class GEOSCoordSeqTest(SimpleTestCase):
    def test_getitem(self):
        coord_seq = LineString([(x, x) for x in range(2)]).coord_seq
        for i in (0, 1):
            with self.subTest(i):
                self.assertEqual(coord_seq[i], (i, i))
        for i in (-3, 10):
            msg = f"Invalid GEOS Geometry index: {i}"
            with self.subTest(i):
                with self.assertRaisesMessage(IndexError, msg):
                    coord_seq[i]

    @skipIf(geos_version_tuple() < (3, 14), "GEOS M support requires 3.14+")
    def test_has_m(self):
        geom = GEOSGeometry("POINT ZM (1 2 3 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=True)
        self.assertIs(coord_seq.hasm, True)

        geom = GEOSGeometry("POINT Z (1 2 3)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=True)
        self.assertIs(coord_seq.hasm, False)

        geom = GEOSGeometry("POINT M (1 2 3)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=False)
        self.assertIs(coord_seq.hasm, True)

    @skipIf(geos_version_tuple() < (3, 14), "GEOS M support requires 3.14+")
    def test_get_set_m(self):
        geom = GEOSGeometry("POINT ZM (1 2 3 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=True)
        self.assertEqual(coord_seq.tuple, (1, 2, 3, 4))
        self.assertEqual(coord_seq.getM(0), 4)
        coord_seq.setM(0, 10)
        self.assertEqual(coord_seq.tuple, (1, 2, 3, 10))
        self.assertEqual(coord_seq.getM(0), 10)

        geom = GEOSGeometry("POINT M (1 2 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=False)
        self.assertEqual(coord_seq.tuple, (1, 2, 4))
        self.assertEqual(coord_seq.getM(0), 4)
        coord_seq.setM(0, 10)
        self.assertEqual(coord_seq.tuple, (1, 2, 10))
        self.assertEqual(coord_seq.getM(0), 10)
        self.assertIs(math.isnan(coord_seq.getZ(0)), True)

    @skipIf(geos_version_tuple() < (3, 14), "GEOS M support requires 3.14+")
    def test_setitem(self):
        geom = GEOSGeometry("POINT ZM (1 2 3 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=True)
        coord_seq[0] = (10, 20, 30, 40)
        self.assertEqual(coord_seq.tuple, (10, 20, 30, 40))

        geom = GEOSGeometry("POINT M (1 2 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=False)
        coord_seq[0] = (10, 20, 40)
        self.assertEqual(coord_seq.tuple, (10, 20, 40))
        self.assertEqual(coord_seq.getM(0), 40)
        self.assertIs(math.isnan(coord_seq.getZ(0)), True)

    @skipIf(geos_version_tuple() < (3, 14), "GEOS M support requires 3.14+")
    def test_kml_m_dimension(self):
        geom = GEOSGeometry("POINT ZM (1 2 3 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=True)
        self.assertEqual(coord_seq.kml, "<coordinates>1.0,2.0,3.0</coordinates>")
        geom = GEOSGeometry("POINT M (1 2 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=False)
        self.assertEqual(coord_seq.kml, "<coordinates>1.0,2.0,0</coordinates>")

    @skipIf(geos_version_tuple() < (3, 14), "GEOS M support requires 3.14+")
    def test_clone_m_dimension(self):
        geom = GEOSGeometry("POINT ZM (1 2 3 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=True)
        clone = coord_seq.clone()
        self.assertEqual(clone.tuple, (1, 2, 3, 4))
        self.assertIs(clone.hasz, True)
        self.assertIs(clone.hasm, True)

        geom = GEOSGeometry("POINT M (1 2 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=False)
        clone = coord_seq.clone()
        self.assertEqual(clone.tuple, (1, 2, 4))
        self.assertIs(clone.hasz, False)
        self.assertIs(clone.hasm, True)

    @skipIf(geos_version_tuple() < (3, 14), "GEOS M support requires 3.14+")
    def test_dims(self):
        geom = GEOSGeometry("POINT ZM (1 2 3 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=True)
        self.assertEqual(coord_seq.dims, 4)

        geom = GEOSGeometry("POINT M (1 2 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=False)
        self.assertEqual(coord_seq.dims, 3)

        geom = GEOSGeometry("POINT Z (1 2 3)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=True)
        self.assertEqual(coord_seq.dims, 3)

        geom = GEOSGeometry("POINT (1 2)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=False)
        self.assertEqual(coord_seq.dims, 2)

    def test_size(self):
        geom = GEOSGeometry("POINT (1 2)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=False)
        self.assertEqual(coord_seq.size, 1)

        geom = GEOSGeometry("POINT M (1 2 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=False)
        self.assertEqual(coord_seq.size, 1)

    @skipIf(geos_version_tuple() < (3, 14), "GEOS M support requires 3.14+")
    def test_iscounterclockwise(self):
        geom = GEOSGeometry("LINEARRING ZM (0 0 3 0, 1 0 0 2, 0 1 1 3, 0 0 3 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=True)
        self.assertEqual(
            coord_seq.tuple,
            (
                (0.0, 0.0, 3.0, 0.0),
                (1.0, 0.0, 0.0, 2.0),
                (0.0, 1.0, 1.0, 3.0),
                (0.0, 0.0, 3.0, 4.0),
            ),
        )
        self.assertIs(coord_seq.is_counterclockwise, True)

    def test_m_support_error(self):
        geom = GEOSGeometry("POINT M (1 2 4)")
        coord_seq = GEOSCoordSeq(capi.get_cs(geom.ptr), z=True)
        msg = "GEOSCoordSeq with an M dimension requires GEOS 3.14+."

        # mock geos_version_tuple to be 3.13.13
        with patch(
            "django.contrib.gis.geos.coordseq.geos_version_tuple",
            return_value=(3, 13, 13),
        ):
            with self.assertRaisesMessage(NotImplementedError, msg):
                coord_seq.hasm



import datetime
import unittest

from django import template
from django.contrib.admin import ModelAdmin
from django.contrib.admin.templatetags.admin_filters import truncated_unordered_list
from django.contrib.admin.templatetags.admin_list import date_hierarchy
from django.contrib.admin.templatetags.admin_modify import submit_row
from django.contrib.admin.templatetags.base import InclusionAdminNode
from django.contrib.auth import get_permission_codename
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import User
from django.template.base import Token, TokenType
from django.test import RequestFactory, SimpleTestCase, TestCase
from django.urls import reverse
from django.utils.version import PY314

from .admin import ArticleAdmin, site
from .models import Article, Question
from .tests import AdminViewBasicTestCase, get_perm


class TruncatedUnorderedListTests(SimpleTestCase):
    def test_no_max_items(self):
        result = truncated_unordered_list(["item 1", "item 2"], None)
        self.assertEqual(result, ("\t<li>item 1</li>\n" "\t<li>item 2</li>"))
        self.assertNotIn("more object", result)

    def test_max_items_zero(self):
        result = truncated_unordered_list(["a", "b", "c"], 0)
        self.assertEqual(result, "")
        self.assertNotIn("more object", result)

    def test_flat_list_truncated(self):
        self.assertEqual(
            truncated_unordered_list(["a", "b", "c", "d", "e"], 3),
            (
                "\t<li>a</li>\n"
                "\t<li>b</li>\n"
                "\t<li>c</li>\n"
                "\t<li>…and 2 more objects.</li>"
            ),
        )

    def test_nested_two_levels_truncated(self):
        self.assertEqual(
            truncated_unordered_list(["a", ["a1", "a2"], "b", "c", "d", "e"], 3),
            (
                "\t<li>a\n"
                "\t<ul>\n"
                "\t\t<li>a1</li>\n"
                "\t\t<li>a2</li>\n"
                "\t</ul>\n"
                "\t</li>\n"
                "\t<li>…and 4 more objects.</li>"
            ),
        )

    def test_nested_and_top_level_truncated(self):
        self.assertEqual(
            truncated_unordered_list(
                ["a", ["n1", "n2", "n3", "n4", "n5"], "b", "c", "d", "e"],
                3,
            ),
            (
                "\t<li>a\n"
                "\t<ul>\n"
                "\t\t<li>n1</li>\n"
                "\t\t<li>n2</li>\n"
                "\t</ul>\n"
                "\t</li>\n"
                "\t<li>…and 7 more objects.</li>"
            ),
        )

    def test_nested_three_levels_truncated(self):
        self.assertEqual(
            truncated_unordered_list(["a", ["a1", ["a1x"]], "b", "c", "d", "e"], 3),
            (
                "\t<li>a\n"
                "\t<ul>\n"
                "\t\t<li>a1\n"
                "\t\t<ul>\n"
                "\t\t\t<li>a1x</li>\n"
                "\t\t</ul>\n"
                "\t\t</li>\n"
                "\t</ul>\n"
                "\t</li>\n"
                "\t<li>…and 4 more objects.</li>"
            ),
        )

    def test_max_items_equal_to_length(self):
        self.assertEqual(
            truncated_unordered_list(["a", "b", "c"], 3),
            ("\t<li>a</li>\n" "\t<li>b</li>\n" "\t<li>c</li>"),
        )

    def test_max_items_greater_than_length(self):
        self.assertEqual(
            truncated_unordered_list(["a", "b"], 10),
            ("\t<li>a</li>\n" "\t<li>b</li>"),
        )

    def test_truncated_single_remaining(self):
        self.assertEqual(
            truncated_unordered_list(["a", "b", "c"], 2),
            ("\t<li>a</li>\n" "\t<li>b</li>\n" "\t<li>…and 1 more object.</li>"),
        )

    def test_autoescape(self):
        self.assertEqual(
            truncated_unordered_list(["<a>item</a>", "safe"], 1),
            ("\t<li>&lt;a&gt;item&lt;/a&gt;</li>\n" "\t<li>…and 1 more object.</li>"),
        )

    def test_autoescape_off(self):
        self.assertEqual(
            truncated_unordered_list(["<a>item</a>", "safe"], 1, autoescape=False),
            ("\t<li><a>item</a></li>\n" "\t<li>…and 1 more object.</li>"),
        )

    def test_empty_list(self):
        self.assertEqual(truncated_unordered_list([], 5), "")

    def test_template_rendering(self):
        t = template.Template(
            "{% load admin_filters %}{{ items|truncated_unordered_list:2 }}"
        )
        result = t.render(template.Context({"items": ["a", "b", "c"]}))
        self.assertIn("<li>a</li>", result)
        self.assertIn("<li>b</li>", result)
        self.assertIn("<li>…and 1 more object.</li>", result)
        self.assertNotIn("<li>c</li>", result)


class AdminTemplateTagsTest(AdminViewBasicTestCase):
    request_factory = RequestFactory()

    def test_submit_row(self):
        """
        submit_row template tag should pass whole context.
        """
        request = self.request_factory.get(
            reverse("admin:auth_user_change", args=[self.superuser.pk])
        )
        request.user = self.superuser
        admin = UserAdmin(User, site)
        extra_context = {"extra": True}
        response = admin.change_view(
            request, str(self.superuser.pk), extra_context=extra_context
        )
        template_context = submit_row(response.context_data)
        self.assertIs(template_context["extra"], True)
        self.assertIs(template_context["show_save"], True)

    def test_submit_row_save_as_new_add_permission_required(self):
        change_user = User.objects.create_user(
            username="change_user", password="secret", is_staff=True
        )
        change_user.user_permissions.add(
            get_perm(User, get_permission_codename("change", User._meta)),
        )
        request = self.request_factory.get(
            reverse("admin:auth_user_change", args=[self.superuser.pk])
        )
        request.user = change_user
        admin = UserAdmin(User, site)
        admin.save_as = True
        response = admin.change_view(request, str(self.superuser.pk))
        template_context = submit_row(response.context_data)
        self.assertIs(template_context["show_save_as_new"], False)

        add_user = User.objects.create_user(
            username="add_user", password="secret", is_staff=True
        )
        add_user.user_permissions.add(
            get_perm(User, get_permission_codename("add", User._meta)),
            get_perm(User, get_permission_codename("change", User._meta)),
        )
        request = self.request_factory.get(
            reverse("admin:auth_user_change", args=[self.superuser.pk])
        )
        request.user = add_user
        response = admin.change_view(request, str(self.superuser.pk))
        template_context = submit_row(response.context_data)
        self.assertIs(template_context["show_save_as_new"], True)

    def test_override_show_save_and_add_another(self):
        request = self.request_factory.get(
            reverse("admin:auth_user_change", args=[self.superuser.pk]),
        )
        request.user = self.superuser
        admin = UserAdmin(User, site)
        for extra_context, expected_flag in (
            ({}, True),  # Default.
            ({"show_save_and_add_another": False}, False),
        ):
            with self.subTest(show_save_and_add_another=expected_flag):
                response = admin.change_view(
                    request,
                    str(self.superuser.pk),
                    extra_context=extra_context,
                )
                template_context = submit_row(response.context_data)
                self.assertIs(
                    template_context["show_save_and_add_another"], expected_flag
                )

    def test_override_change_form_template_tags(self):
        """
        admin_modify template tags follow the standard search pattern
        admin/app_label/model/template.html.
        """
        article = Article.objects.all()[0]
        request = self.request_factory.get(
            reverse("admin:admin_views_article_change", args=[article.pk])
        )
        request.user = self.superuser
        admin = ArticleAdmin(Article, site)
        extra_context = {"show_publish": True, "extra": True}
        response = admin.change_view(
            request, str(article.pk), extra_context=extra_context
        )
        response.render()
        self.assertIs(response.context_data["show_publish"], True)
        self.assertIs(response.context_data["extra"], True)
        self.assertContains(response, 'name="_save"')
        self.assertContains(response, 'name="_publish"')
        self.assertContains(response, "override-change_form_object_tools")
        self.assertContains(response, "override-prepopulated_fields_js")

    def test_override_change_list_template_tags(self):
        """
        admin_list template tags follow the standard search pattern
        admin/app_label/model/template.html.
        """
        request = self.request_factory.get(
            reverse("admin:admin_views_article_changelist")
        )
        request.user = self.superuser
        admin = ArticleAdmin(Article, site)
        admin.date_hierarchy = "date"
        admin.search_fields = ("title", "content")
        response = admin.changelist_view(request)
        response.render()
        self.assertContains(response, "override-actions")
        self.assertContains(response, "override-change_list_object_tools")
        self.assertContains(response, "override-change_list_results")
        self.assertContains(response, "override-date_hierarchy")
        self.assertContains(response, "override-pagination")
        self.assertContains(response, "override-search_form")

    @unittest.skipUnless(PY314, "Deferred annotations are Python 3.14+ only")
    def test_inclusion_admin_node_deferred_annotation(self):
        def action(arg: SomeType = None):  # NOQA: F821
            pass

        # This used to raise TypeError via the underlying call to
        # inspect.getfullargspec(), which is not ready for deferred
        # evaluation of annotations.
        InclusionAdminNode(
            "test",
            parser=object(),
            token=Token(token_type=TokenType.TEXT, contents="a"),
            func=action,
            template_name="test.html",
            takes_context=False,
        )


class DateHierarchyTests(TestCase):
    factory = RequestFactory()

    @classmethod
    def setUpTestData(cls):
        cls.superuser = User.objects.create_superuser(
            username="super", password="secret", email="super@example.com"
        )

    def test_choice_links(self):
        modeladmin = ModelAdmin(Question, site)
        modeladmin.date_hierarchy = "posted"

        posted_dates = (
            datetime.date(2017, 10, 1),
            datetime.date(2017, 10, 1),
            datetime.date(2017, 12, 15),
            datetime.date(2017, 12, 15),
            datetime.date(2017, 12, 31),
            datetime.date(2018, 2, 1),
        )
        Question.objects.bulk_create(
            Question(question="q", posted=posted) for posted in posted_dates
        )

        tests = (
            ({}, [["year=2017"], ["year=2018"]]),
            ({"year": 2016}, []),
            ({"year": 2017}, [["month=10", "year=2017"], ["month=12", "year=2017"]]),
            ({"year": 2017, "month": 9}, []),
            (
                {"year": 2017, "month": 12},
                [
                    ["day=15", "month=12", "year=2017"],
                    ["day=31", "month=12", "year=2017"],
                ],
            ),
        )
        for query, expected_choices in tests:
            with self.subTest(query=query):
                query = {"posted__%s" % q: val for q, val in query.items()}
                request = self.factory.get("/", query)
                request.user = self.superuser
                changelist = modeladmin.get_changelist_instance(request)
                spec = date_hierarchy(changelist)
                choices = [choice["link"] for choice in spec["choices"]]
                expected_choices = [
                    "&".join("posted__%s" % c for c in choice)
                    for choice in expected_choices
                ]
                expected_choices = [
                    ("?" + choice) if choice else "" for choice in expected_choices
                ]
                self.assertEqual(choices, expected_choices)

    def test_choice_links_datetime(self):
        modeladmin = ModelAdmin(Question, site)
        modeladmin.date_hierarchy = "expires"
        Question.objects.bulk_create(
            [
                Question(question="q1", expires=datetime.datetime(2017, 10, 1)),
                Question(question="q2", expires=datetime.datetime(2017, 10, 1)),
                Question(question="q3", expires=datetime.datetime(2017, 12, 15)),
                Question(question="q4", expires=datetime.datetime(2017, 12, 15)),
                Question(question="q5", expires=datetime.datetime(2017, 12, 31)),
                Question(question="q6", expires=datetime.datetime(2018, 2, 1)),
            ]
        )
        tests = [
            ({}, [["year=2017"], ["year=2018"]]),
            ({"year": 2016}, []),
            (
                {"year": 2017},
                [
                    ["month=10", "year=2017"],
                    ["month=12", "year=2017"],
                ],
            ),
            ({"year": 2017, "month": 9}, []),
            (
                {"year": 2017, "month": 12},
                [
                    ["day=15", "month=12", "year=2017"],
                    ["day=31", "month=12", "year=2017"],
                ],
            ),
        ]
        for query, expected_choices in tests:
            with self.subTest(query=query):
                query = {"expires__%s" % q: val for q, val in query.items()}
                request = self.factory.get("/", query)
                request.user = self.superuser
                changelist = modeladmin.get_changelist_instance(request)
                spec = date_hierarchy(changelist)
                choices = [choice["link"] for choice in spec["choices"]]
                expected_choices = [
                    "?" + "&".join("expires__%s" % c for c in choice)
                    for choice in expected_choices
                ]
                self.assertEqual(choices, expected_choices)



import django.contrib.sites.managers
import django.db.models.manager
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sites", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomArticle",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=50)),
                (
                    "places_this_article_should_appear",
                    models.ForeignKey("sites.Site", models.CASCADE),
                ),
            ],
            options={
                "abstract": False,
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                (
                    "on_site",
                    django.contrib.sites.managers.CurrentSiteManager(
                        "places_this_article_should_appear"
                    ),
                ),
            ],
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="ExclusiveArticle",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=50)),
                ("site", models.ForeignKey("sites.Site", models.CASCADE)),
            ],
            options={
                "abstract": False,
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
            ],
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="SyndicatedArticle",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=50)),
                ("sites", models.ManyToManyField("sites.Site")),
            ],
            options={
                "abstract": False,
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
            ],
            bases=(models.Model,),
        ),
    ]



from django.core.mail.backends.base import BaseEmailBackend


class EmailBackend(BaseEmailBackend):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.test_outbox = []

    def send_messages(self, email_messages):
        # Messages are stored in an instance variable for testing.
        self.test_outbox.extend(email_messages)
        return len(email_messages)


class OptionsCapturingBackend(BaseEmailBackend):
    """Capture init kwargs and sent messages for use in test assertions.

    Test cases using this backend _must_ ensure reset() is called::

        def test_something(self):
            self.addCleanup(OptionsCapturingBackend.reset)
            ...

    Failing to call reset() will cause unexpected behavior in other tests that
    use the OptionsCapturingBackend.
    """

    init_kwargs = []
    sent_messages = []

    @classmethod
    def reset(cls):
        cls.init_kwargs = []
        cls.sent_messages = []

    def __init__(self, **kwargs):
        self.init_kwargs.append(kwargs)
        if "alias" in kwargs:
            super().__init__(alias=kwargs["alias"])
        else:
            super().__init__()

    def send_messages(self, email_messages):
        self.sent_messages.extend(email_messages)
        return len(email_messages)


class FailingEmailBackend(OptionsCapturingBackend):
    """Raise on send_messages(), or do nothing if fail_silently is set.

    Test cases using this backend _must_ ensure reset() is called::

        def test_something(self):
            self.addCleanup(FailingEmailBackend.reset)
            ...

    Failing to call reset() will cause unexpected behavior in other tests that
    use the FailingEmailBackend.
    """

    init_kwargs = []
    sent_messages = []

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.fail_silently = kwargs.get("fail_silently", False)

    def send_messages(self, email_messages):
        if self.fail_silently:
            return 0
        raise ValueError("FailingEmailBackend is doomed to fail.")



"""
Testing some internals of the template processing.
These are *not* examples to be copied in user code.
"""

import unittest

from django.template import Library, TemplateSyntaxError
from django.template.base import (
    FilterExpression,
    Lexer,
    Parser,
    Token,
    TokenType,
    Variable,
    VariableDoesNotExist,
)
from django.template.defaultfilters import register as filter_library
from django.test import SimpleTestCase
from django.utils.version import PY314


class ParserTests(SimpleTestCase):
    def test_token_smart_split(self):
        """
        #7027 -- _() syntax should work with spaces
        """
        token = Token(
            TokenType.BLOCK, 'sometag _("Page not found") value|yesno:_("yes,no")'
        )
        split = token.split_contents()
        self.assertEqual(
            split, ["sometag", '_("Page not found")', 'value|yesno:_("yes,no")']
        )

    def test_repr(self):
        token = Token(TokenType.BLOCK, "some text")
        self.assertEqual(repr(token), '<Block token: "some text...">')
        parser = Parser([token], builtins=[filter_library])
        self.assertEqual(
            repr(parser),
            '<Parser tokens=[<Block token: "some text...">]>',
        )
        filter_expression = FilterExpression("news|upper", parser)
        self.assertEqual(repr(filter_expression), "<FilterExpression 'news|upper'>")
        lexer = Lexer("{% for i in 1 %}{{ a }}\n{% endfor %}")
        self.assertEqual(
            repr(lexer),
            '<Lexer template_string="{% for i in 1 %}{{ a...", verbatim=False>',
        )

    def test_filter_parsing(self):
        c = {"article": {"section": "News"}}
        p = Parser("", builtins=[filter_library])

        def fe_test(s, val):
            self.assertEqual(FilterExpression(s, p).resolve(c), val)

        fe_test("article.section", "News")
        fe_test("article.section|upper", "NEWS")
        fe_test('"News"', "News")
        fe_test("'News'", "News")
        fe_test(r'"Some \"Good\" News"', 'Some "Good" News')
        fe_test(r'"Some \"Good\" News"', 'Some "Good" News')
        fe_test(r"'Some \'Bad\' News'", "Some 'Bad' News")

        fe = FilterExpression(r'"Some \"Good\" News"', p)
        self.assertEqual(fe.filters, [])
        self.assertEqual(fe.var, 'Some "Good" News')

        # Filtered variables should reject access of attributes beginning with
        # underscores.
        msg = (
            "Variables and attributes may not begin with underscores: 'article._hidden'"
        )
        with self.assertRaisesMessage(TemplateSyntaxError, msg):
            FilterExpression("article._hidden|upper", p)

    def test_cannot_parse_characters(self):
        p = Parser("", builtins=[filter_library])
        for filter_expression, characters in [
            ('<>|default:"Default"|upper', '|<>||default:"Default"|upper'),
            ("test|<>|upper", "test||<>||upper"),
        ]:
            with self.subTest(filter_expression=filter_expression):
                with self.assertRaisesMessage(
                    TemplateSyntaxError,
                    f"Could not parse some characters: {characters}",
                ):
                    FilterExpression(filter_expression, p)

    def test_cannot_find_variable(self):
        p = Parser("", builtins=[filter_library])
        with self.assertRaisesMessage(
            TemplateSyntaxError,
            'Could not find variable at start of |default:"Default"',
        ):
            FilterExpression('|default:"Default"', p)

    def test_variable_parsing(self):
        c = {"article": {"section": "News"}}
        self.assertEqual(Variable("article.section").resolve(c), "News")
        self.assertEqual(Variable('"News"').resolve(c), "News")
        self.assertEqual(Variable("'News'").resolve(c), "News")

        # Translated strings are handled correctly.
        self.assertEqual(Variable("_(article.section)").resolve(c), "News")
        self.assertEqual(Variable('_("Good News")').resolve(c), "Good News")
        self.assertEqual(Variable("_('Better News')").resolve(c), "Better News")

        # Escaped quotes work correctly as well.
        self.assertEqual(
            Variable(r'"Some \"Good\" News"').resolve(c), 'Some "Good" News'
        )
        self.assertEqual(
            Variable(r"'Some \'Better\' News'").resolve(c), "Some 'Better' News"
        )

        # Variables should reject access of attributes and variables beginning
        # with underscores.
        for name in ["article._hidden", "_article"]:
            msg = f"Variables and attributes may not begin with underscores: '{name}'"
            with self.assertRaisesMessage(TemplateSyntaxError, msg):
                Variable(name)

        # Variables should raise on non string type
        with self.assertRaisesMessage(
            TypeError, "Variable must be a string or number, got <class 'dict'>"
        ):
            Variable({})

        # Variables should raise when invalid characters in name.
        for c in ["+", "-"]:
            with self.subTest(invalid_character=c):
                variable_name = f"variable{c}name"
                with self.assertRaisesMessage(
                    TemplateSyntaxError,
                    f"Invalid character ('{c}') in variable name: '{variable_name}'",
                ):
                    Variable(variable_name)

    def test_filter_args_count(self):
        parser = Parser("")
        register = Library()

        @register.filter
        def no_arguments(value):
            pass

        @register.filter
        def one_argument(value, arg):
            pass

        @register.filter
        def one_opt_argument(value, arg=False):
            pass

        @register.filter
        def two_arguments(value, arg, arg2):
            pass

        @register.filter
        def two_one_opt_arg(value, arg, arg2=False):
            pass

        parser.add_library(register)
        for expr in (
            '1|no_arguments:"1"',
            "1|two_arguments",
            '1|two_arguments:"1"',
            "1|two_one_opt_arg",
        ):
            with self.assertRaises(TemplateSyntaxError):
                FilterExpression(expr, parser)
        for expr in (
            # Correct number of arguments
            "1|no_arguments",
            '1|one_argument:"1"',
            # One optional
            "1|one_opt_argument",
            '1|one_opt_argument:"1"',
            # Not supplying all
            '1|two_one_opt_arg:"1"',
        ):
            FilterExpression(expr, parser)

    def test_filter_numeric_argument_parsing(self):
        p = Parser("", builtins=[filter_library])

        # Values that resolve to a numeric literal.
        cases = {
            "5": 5,
            "-5": -5,
            "5.2": 5.2,
            ".4": 0.4,
            "5.2e3": 5200.0,  # 5.2 × 10³ = 5200.0.
            "5.2E3": 5200.0,  # Case-insensitive.
            "5.2e-3": 0.0052,  # Negative exponent.
            "-1.5E4": -15000.0,
            "+3.0e2": 300.0,
            ".5e2": 50.0,  # 0.5 × 10² = 50.0
        }
        for num, expected in cases.items():
            with self.subTest(num=num):
                self.assertEqual(FilterExpression(num, p).resolve({}), expected)
                self.assertEqual(
                    FilterExpression(f"0|default:{num}", p).resolve({}), expected
                )

        # Values that are interpreted as names of variables that do not exist.
        invalid_numbers = [
            "abc123",
            "123abc",
            "foo",
            "error",
            "1e",
            "e400",
            "1e.2",
            "1e2.",
            "1e2.0",
            "1e2a",
            "1e2e3",
        ]

        for num in invalid_numbers:
            with self.subTest(num=num):
                self.assertIsNone(
                    FilterExpression(num, p).resolve({}, ignore_failures=True)
                )
                with self.assertRaises(VariableDoesNotExist):
                    FilterExpression(f"0|default:{num}", p).resolve({})

        # Values that are interpreted as an invalid variable name.
        invalid_numbers_and_var_names = [
            "1e-",
            "1e-a",
            "1+1",
            "1-1",
        ]
        for num in invalid_numbers_and_var_names:
            with self.subTest(num=num):
                with self.assertRaises(TemplateSyntaxError):
                    FilterExpression(num, p).resolve({})
                with self.assertRaises(TemplateSyntaxError):
                    FilterExpression(f"0|default:{num}", p).resolve({})

    @unittest.skipUnless(PY314, "Deferred annotations are Python 3.14+ only")
    def test_filter_deferred_annotation(self):
        register = Library()

        @register.filter("example")
        def example_filter(value: str, arg: SomeType):  # NOQA: F821
            return f"{value}_{arg}"

        result = FilterExpression.args_check(
            "example", example_filter, ["extra_example"]
        )

        self.assertIs(result, True)



from collections import deque

from django.db.models import Prefetch, prefetch_related_objects
from django.test import TestCase

from .models import Author, Book, House, Reader, Room


class PrefetchRelatedObjectsTests(TestCase):
    """
    Since prefetch_related_objects() is just the inner part of
    prefetch_related(), only do basic tests to ensure its API hasn't changed.
    """

    @classmethod
    def setUpTestData(cls):
        cls.book1 = Book.objects.create(title="Poems")
        cls.book2 = Book.objects.create(title="Jane Eyre")
        cls.book3 = Book.objects.create(title="Wuthering Heights")
        cls.book4 = Book.objects.create(title="Sense and Sensibility")

        cls.author1 = Author.objects.create(name="Charlotte", first_book=cls.book1)
        cls.author2 = Author.objects.create(name="Anne", first_book=cls.book1)
        cls.author3 = Author.objects.create(name="Emily", first_book=cls.book1)
        cls.author4 = Author.objects.create(name="Jane", first_book=cls.book4)

        cls.book1.authors.add(cls.author1, cls.author2, cls.author3)
        cls.book2.authors.add(cls.author1)
        cls.book3.authors.add(cls.author3)
        cls.book4.authors.add(cls.author4)

        cls.reader1 = Reader.objects.create(name="Amy")
        cls.reader2 = Reader.objects.create(name="Belinda")

        cls.reader1.books_read.add(cls.book1, cls.book4)
        cls.reader2.books_read.add(cls.book2, cls.book4)

        cls.house1 = House.objects.create(name="b1", address="1")
        cls.house2 = House.objects.create(name="b2", address="2")

        cls.room1 = Room.objects.create(name="a1", house=cls.house1)
        cls.room2 = Room.objects.create(name="a2", house=cls.house2)

        cls.house1.main_room = cls.room1
        cls.house1.save()
        cls.house2.main_room = cls.room2
        cls.house2.save()

    def test_unknown(self):
        book1 = Book.objects.get(id=self.book1.id)
        with self.assertRaises(AttributeError):
            prefetch_related_objects([book1], "unknown_attribute")

    def test_m2m_forward(self):
        book1 = Book.objects.get(id=self.book1.id)
        with self.assertNumQueries(1):
            prefetch_related_objects([book1], "authors")

        with self.assertNumQueries(0):
            self.assertCountEqual(
                book1.authors.all(), [self.author1, self.author2, self.author3]
            )

    def test_m2m_reverse(self):
        author1 = Author.objects.get(id=self.author1.id)
        with self.assertNumQueries(1):
            prefetch_related_objects([author1], "books")

        with self.assertNumQueries(0):
            self.assertCountEqual(author1.books.all(), [self.book1, self.book2])

    def test_foreignkey_forward(self):
        authors = list(Author.objects.all())
        with self.assertNumQueries(1) as ctx:
            prefetch_related_objects(authors, "first_book")
        self.assertNotIn("ORDER BY", ctx.captured_queries[0]["sql"])

        with self.assertNumQueries(0):
            [author.first_book for author in authors]

        authors = list(Author.objects.all())
        with self.assertNumQueries(1) as ctx:
            prefetch_related_objects(
                authors,
                Prefetch("first_book", queryset=Book.objects.order_by("-title")),
            )
        self.assertNotIn("ORDER BY", ctx.captured_queries[0]["sql"])

    def test_foreignkey_reverse(self):
        books = list(Book.objects.all())
        with self.assertNumQueries(1) as ctx:
            prefetch_related_objects(books, "first_time_authors")
        self.assertIn("ORDER BY", ctx.captured_queries[0]["sql"])

        with self.assertNumQueries(0):
            [list(book.first_time_authors.all()) for book in books]

        books = list(Book.objects.all())
        with self.assertNumQueries(1) as ctx:
            prefetch_related_objects(
                books,
                Prefetch(
                    "first_time_authors",
                    queryset=Author.objects.order_by("-name"),
                ),
            )
        self.assertIn("ORDER BY", ctx.captured_queries[0]["sql"])

    def test_one_to_one_forward(self):
        houses = list(House.objects.all())
        with self.assertNumQueries(1) as ctx:
            prefetch_related_objects(houses, "main_room")
        self.assertNotIn("ORDER BY", ctx.captured_queries[0]["sql"])

        with self.assertNumQueries(0):
            [house.main_room for house in houses]

        houses = list(House.objects.all())
        with self.assertNumQueries(1) as ctx:
            prefetch_related_objects(
                houses,
                Prefetch("main_room", queryset=Room.objects.order_by("-name")),
            )
        self.assertNotIn("ORDER BY", ctx.captured_queries[0]["sql"])

    def test_one_to_one_reverse(self):
        rooms = list(Room.objects.all())
        with self.assertNumQueries(1) as ctx:
            prefetch_related_objects(rooms, "main_room_of")
        self.assertNotIn("ORDER BY", ctx.captured_queries[0]["sql"])

        with self.assertNumQueries(0):
            [room.main_room_of for room in rooms]

        rooms = list(Room.objects.all())
        with self.assertNumQueries(1) as ctx:
            prefetch_related_objects(
                rooms,
                Prefetch("main_room_of", queryset=House.objects.order_by("-name")),
            )
        self.assertNotIn("ORDER BY", ctx.captured_queries[0]["sql"])

    def test_m2m_then_m2m(self):
        """A m2m can be followed through another m2m."""
        authors = list(Author.objects.all())
        with self.assertNumQueries(2):
            prefetch_related_objects(authors, "books__read_by")

        with self.assertNumQueries(0):
            self.assertEqual(
                [
                    [[str(r) for r in b.read_by.all()] for b in a.books.all()]
                    for a in authors
                ],
                [
                    [["Amy"], ["Belinda"]],  # Charlotte - Poems, Jane Eyre
                    [["Amy"]],  # Anne - Poems
                    [["Amy"], []],  # Emily - Poems, Wuthering Heights
                    [["Amy", "Belinda"]],  # Jane - Sense and Sense
                ],
            )

    def test_prefetch_object(self):
        book1 = Book.objects.get(id=self.book1.id)
        with self.assertNumQueries(1):
            prefetch_related_objects([book1], Prefetch("authors"))

        with self.assertNumQueries(0):
            self.assertCountEqual(
                book1.authors.all(), [self.author1, self.author2, self.author3]
            )

    def test_prefetch_object_twice(self):
        book1 = Book.objects.get(id=self.book1.id)
        book2 = Book.objects.get(id=self.book2.id)
        with self.assertNumQueries(1):
            prefetch_related_objects([book1], Prefetch("authors"))
        with self.assertNumQueries(1):
            prefetch_related_objects([book1, book2], Prefetch("authors"))
        with self.assertNumQueries(0):
            self.assertCountEqual(book2.authors.all(), [self.author1])

    def test_prefetch_object_to_attr(self):
        book1 = Book.objects.get(id=self.book1.id)
        with self.assertNumQueries(1):
            prefetch_related_objects(
                [book1], Prefetch("authors", to_attr="the_authors")
            )

        with self.assertNumQueries(0):
            self.assertCountEqual(
                book1.the_authors, [self.author1, self.author2, self.author3]
            )

    def test_prefetch_object_to_attr_twice(self):
        book1 = Book.objects.get(id=self.book1.id)
        book2 = Book.objects.get(id=self.book2.id)
        with self.assertNumQueries(1):
            prefetch_related_objects(
                [book1],
                Prefetch("authors", to_attr="the_authors"),
            )
        with self.assertNumQueries(1):
            prefetch_related_objects(
                [book1, book2],
                Prefetch("authors", to_attr="the_authors"),
            )
        with self.assertNumQueries(0):
            self.assertCountEqual(book2.the_authors, [self.author1])

    def test_prefetch_queryset(self):
        book1 = Book.objects.get(id=self.book1.id)
        with self.assertNumQueries(1):
            prefetch_related_objects(
                [book1],
                Prefetch(
                    "authors",
                    queryset=Author.objects.filter(
                        id__in=[self.author1.id, self.author2.id]
                    ),
                ),
            )

        with self.assertNumQueries(0):
            self.assertCountEqual(book1.authors.all(), [self.author1, self.author2])

    def test_prefetch_related_objects_with_various_iterables(self):
        book = self.book1

        class MyIterable:
            def __iter__(self):
                yield book

        cases = {
            "set": {book},
            "tuple": (book,),
            "dict_values": {"a": book}.values(),
            "frozenset": frozenset([book]),
            "deque": deque([book]),
            "custom iterator": MyIterable(),
        }
        for case_type, case in cases.items():
            with self.subTest(case=case_type):
                # Clear the prefetch cache.
                book._prefetched_objects_cache = {}
                with self.assertNumQueries(1):
                    prefetch_related_objects(case, "authors")
                with self.assertNumQueries(0):
                    self.assertCountEqual(
                        book.authors.all(), [self.author1, self.author2, self.author3]
                    )

    def test_prefetch_related_objects_empty(self):
        prefetch_related_objects([], "authors")



# RemovedInDjango70Warning: This entire file.
import re
import types
import warnings
from contextlib import contextmanager
from email.mime.text import MIMEText
from unittest import mock

import django.conf
import django.utils.timezone
from django.conf import LazySettings
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import (
    EmailAlternative,
    EmailAttachment,
    EmailMessage,
    EmailMultiAlternatives,
    get_connection,
    mailers,
)
from django.core.mail.deprecation import NO_DEFAULT_MAILER_WARNING
from django.core.mail.message import forbid_multi_line_headers, sanitize_address
from django.test import SimpleTestCase, ignore_warnings, override_settings
from django.utils.deprecation import RemovedInDjango70Warning

from . import override_deprecated_email_settings
from .tests import MailTestsMixin


class DeprecationWarningTests(MailTestsMixin, SimpleTestCase):
    def test_deprecated_on_import(self):
        """
        These items are not typically called from user code,
        so generate deprecation warnings immediately at the time
        they are imported from django.core.mail.
        """
        cases = [
            # name, msg
            (
                "BadHeaderError",
                "BadHeaderError is deprecated. Replace with ValueError.",
            ),
            (
                "SafeMIMEText",
                "SafeMIMEText is deprecated. The return value of"
                " EmailMessage.message() is an email.message.EmailMessage.",
            ),
            (
                "SafeMIMEMultipart",
                "SafeMIMEMultipart is deprecated. The return value of"
                " EmailMessage.message() is an email.message.EmailMessage.",
            ),
        ]
        for name, msg in cases:
            with self.subTest(name=name):
                with self.assertWarnsMessage(RemovedInDjango70Warning, msg):
                    __import__("django.core.mail", fromlist=[name])

    def test_sanitize_address_deprecated(self):
        msg = (
            "The internal API sanitize_address() is deprecated."
            " Python's modern email API (with email.message.EmailMessage or"
            " email.policy.default) will handle most required validation and"
            " encoding. Use Python's email.headerregistry.Address to construct"
            " formatted addresses from component parts."
        )
        with self.assertWarnsMessage(RemovedInDjango70Warning, msg):
            sanitize_address("to@example.com", "ascii")

    def test_forbid_multi_line_headers_deprecated(self):
        msg = (
            "The internal API forbid_multi_line_headers() is deprecated."
            " Python's modern email API (with email.message.EmailMessage or"
            " email.policy.default) will reject multi-line headers."
        )
        with self.assertWarnsMessage(RemovedInDjango70Warning, msg):
            forbid_multi_line_headers("To", "to@example.com", "ascii")


class UndocumentedFeatureErrorTests(SimpleTestCase):
    """
    These undocumented features were removed without going through deprecation.
    In case they were being used, they now raise errors.
    """

    def test_undocumented_mixed_subtype(self):
        """
        Trying to use the previously undocumented, now unsupported
        EmailMessage.mixed_subtype causes an error.
        """
        msg = (
            "EmailMessage no longer supports"
            " the undocumented `mixed_subtype` attribute"
        )
        email = EmailMessage(
            attachments=[EmailAttachment(None, b"GIF89a...", "image/gif")]
        )
        email.mixed_subtype = "related"
        with self.assertRaisesMessage(AttributeError, msg):
            email.message()

    def test_undocumented_alternative_subtype(self):
        """
        Trying to use the previously undocumented, now unsupported
        EmailMultiAlternatives.alternative_subtype causes an error.
        """
        msg = (
            "EmailMultiAlternatives no longer supports"
            " the undocumented `alternative_subtype` attribute"
        )
        email = EmailMultiAlternatives(
            alternatives=[EmailAlternative("", "text/plain")]
        )
        email.alternative_subtype = "multilingual"
        with self.assertRaisesMessage(AttributeError, msg):
            email.message()

    def test_undocumented_get_connection_override_no_longer_supported(self):

        class CustomEmailMessage(EmailMessage):
            def get_connection(self, fail_silently=False):
                return None

        email = CustomEmailMessage(to=["to@example.com"])

        msg = (
            "EmailMessage no longer supports the undocumented "
            "get_connection() method."
        )
        with self.assertRaisesMessage(AttributeError, msg):
            email.send()


@ignore_warnings(category=RemovedInDjango70Warning)
class DeprecatedCompatibilityTests(SimpleTestCase):
    def test_bad_header_error(self):
        """
        Existing code that catches deprecated BadHeaderError should be
        compatible with modern email (which raises ValueError instead).
        """
        from django.core.mail import BadHeaderError

        with self.assertRaises(BadHeaderError):
            EmailMessage(subject="Bad\r\nHeader").message()

    def test_attachments_mimebase_in_constructor(self):
        txt = MIMEText("content1")
        msg = EmailMessage(attachments=[txt])
        payload = msg.message().get_payload()
        self.assertEqual(payload[0], txt)


class DeprecatedEmailSettingsTests(SimpleTestCase):
    """Deprecations and compatibility errors related to MAILERS."""

    deprecated_setting_defaults = {
        "EMAIL_BACKEND": "django.core.mail.backends.smtp.EmailBackend",
        "EMAIL_HOST": "localhost",
        "EMAIL_PORT": 25,
        "EMAIL_HOST_USER": "",
        "EMAIL_HOST_PASSWORD": "",
        "EMAIL_USE_TLS": False,
        "EMAIL_USE_SSL": False,
        "EMAIL_SSL_CERTFILE": None,
        "EMAIL_SSL_KEYFILE": None,
        "EMAIL_TIMEOUT": None,
        # EMAIL_FILE_PATH does not have a default.
    }

    deprecated_settings = deprecated_setting_defaults.keys() | {"EMAIL_FILE_PATH"}

    # Tests for defining settings must cover three separate cases, which go
    # through different code paths in django.conf:
    # - settings module (settings.py; LazySettings wraps Settings)
    # - settings.configure() (LazySettings wraps UserSettingsHolder)
    # - override_settings() (a.k.a. SimpleTestCase.settings(); temporarily
    #   inserts a UserSettingsHolder into the current LazySettings)
    #
    # A settings module must be simulated in these tests. (The real settings
    # module can't be modified, and override_settings() isn't the same as using
    # a settings module.) To do that:
    # - Optionally use a self.mock_settings_module() context to populate a
    #   simulated settings.py.
    # - Call self.init_simulated_settings() to initialize settings from a
    #   module (the mock_settings_module() if active, else the real one) and
    #   return a settings object equivalent to django.conf.settings.

    def mock_settings_module(self, **settings):
        settings_module = types.ModuleType("mocked_settings")
        for name, value in settings.items():
            setattr(settings_module, name, value)
        # Patch the settings module import in Settings.__init__() to
        # substitute the mocked settings.
        return mock.patch(
            "django.conf.importlib.import_module",
            autospec=True,
            return_value=settings_module,
        )

    def init_simulated_settings(self):
        settings = LazySettings()
        # Trigger LazySettings._setup() *from within Django*. (In real use,
        # the first settings access is often iter(db.connections) in
        # run_checks() or similar. But any settings access will work, and
        # it's hard to mock db.connections due to @cached_property() usage.)
        with mock.patch.object(django.utils.timezone, "settings", settings):
            django.utils.timezone.now()  # Reads settings.USE_TZ.
        return settings

    @contextmanager
    def assertNotWarnsMessage(self, category, message):
        with warnings.catch_warnings(record=True) as caught_warnings:
            warnings.filterwarnings(
                "always", category=category, message=rf".*{re.escape(message)}"
            )
            yield caught_warnings
        self.assertEqual([str(warning) for warning in caught_warnings], [])

    def assertHasOnlyDefaultEmailSettings(self, settings, msg=None):
        non_default_settings = [
            name for name in self.deprecated_settings if settings.is_overridden(name)
        ]
        if hasattr(settings, "MAILERS"):
            non_default_settings.append("MAILERS")
        self.assertEqual(non_default_settings, [], msg=msg)

    def test_warn_when_defining_deprecated_settings(self):
        for name in self.deprecated_settings:
            msg = (
                f"The {name} setting is deprecated. Migrate to "
                "MAILERS before Django 7.0."
            )
            settings = {name: "foo"}
            with self.subTest(name=name):
                with (
                    self.subTest("settings module"),
                    self.mock_settings_module(**settings),
                    self.assertWarnsMessage(RemovedInDjango70Warning, msg),
                ):
                    self.init_simulated_settings()
                with (
                    self.subTest("settings.configure()"),
                    self.assertWarnsMessage(RemovedInDjango70Warning, msg),
                ):
                    LazySettings().configure(**settings)
                with (
                    self.subTest("override_settings()"),
                    self.assertWarnsMessage(RemovedInDjango70Warning, msg),
                ):
                    with override_settings(**settings):
                        pass

    def test_multiple_deprecated_settings_are_all_reported(self):
        msg_re = r"The EMAIL_(BACKEND|HOST|PORT) setting is deprecated."
        settings = {"EMAIL_BACKEND": "foo", "EMAIL_HOST": "bar", "EMAIL_PORT": 2525}
        expected_warning_count = len(settings)

        with self.subTest("settings module"), self.mock_settings_module(**settings):
            with self.assertWarnsRegex(RemovedInDjango70Warning, msg_re) as cm:
                self.init_simulated_settings()
            self.assertEqual(len(cm.warnings), expected_warning_count)

        with self.subTest("settings.configure()"):
            with self.assertWarnsRegex(RemovedInDjango70Warning, msg_re) as cm:
                LazySettings().configure(**settings)
            self.assertEqual(len(cm.warnings), expected_warning_count)

        with self.subTest("override_settings()"):
            with (
                self.assertWarnsRegex(RemovedInDjango70Warning, msg_re) as cm,
                override_settings(**settings),
            ):
                pass
            # override_settings() accesses each setting twice: setattr while
            # enabling and getattr while disabling (for change notification).
            self.assertEqual(len(cm.warnings), 2 * expected_warning_count)

    def test_warn_about_no_default_mailer(self):
        # Test precondition: the real settings object must be default.
        self.assertHasOnlyDefaultEmailSettings(django.conf.settings)

        # The warning is issued if no email-backend-related settings are
        # defined, but only when email is sent. Any attempt to send email
        # should invoke get_connection() or mailers.create_connection().
        msg = NO_DEFAULT_MAILER_WARNING
        with (
            self.subTest("get_connection()"),
            self.assertWarnsMessage(RemovedInDjango70Warning, msg),
            ignore_warnings(
                category=RemovedInDjango70Warning,
                message=re.escape("get_connection() is deprecated."),
            ),
        ):
            get_connection()
        with (
            self.subTest("mailers.default"),
            self.assertWarnsMessage(RemovedInDjango70Warning, msg),
        ):
            _ = mailers.default

        # The warning is not issued on startup, to avoid creating noise for
        # projects that don't send email at all.
        with self.assertNotWarnsMessage(RemovedInDjango70Warning, msg):
            settings = self.init_simulated_settings()
        self.assertHasOnlyDefaultEmailSettings(settings, msg="invalid test")

    def test_no_default_mailer_warning_if_any_email_setting_defined(self):
        # Test precondition: the real settings object must be default.
        self.assertHasOnlyDefaultEmailSettings(django.conf.settings)

        # The warning from the previous test is not issued if any deprecated
        # email setting is defined (which would result in a startup-time
        # warning about MAILERS) or if MAILERS is defined.
        msg = NO_DEFAULT_MAILER_WARNING
        for name in self.deprecated_settings:
            value = self.deprecated_setting_defaults.get(name, "foo")
            with (
                self.subTest(name=name),
                override_deprecated_email_settings(**{name: value}),
                self.assertNotWarnsMessage(RemovedInDjango70Warning, msg),
            ):
                _ = mailers.default
        with (
            self.subTest(name="MAILERS"),
            self.settings(
                MAILERS={
                    "default": {
                        "BACKEND": "django.core.mail.backends.locmem.EmailBackend"
                    }
                }
            ),
            self.assertNotWarnsMessage(RemovedInDjango70Warning, msg),
        ):
            _ = mailers.default

    def test_deprecated_settings_not_allowed_with_mailers(self):
        for name in self.deprecated_settings:
            msg = (
                "Deprecated email settings are not allowed when "
                f"MAILERS is defined: {name}."
            )
            settings = {name: "foo", "MAILERS": {}}
            with self.subTest(name=name):
                with (
                    self.subTest("settings module"),
                    self.mock_settings_module(**settings),
                    self.assertRaisesMessage(ImproperlyConfigured, msg),
                ):
                    self.init_simulated_settings()
                with (
                    self.subTest("settings.configure()"),
                    self.assertRaisesMessage(ImproperlyConfigured, msg),
                ):
                    LazySettings().configure(**settings)
                # There is intentionally no override_settings() subtest here.
                # override_settings() does not check for settings conflicts.

    def test_warn_when_using_deprecated_settings(self):
        for name in self.deprecated_settings:
            msg = (
                f"The {name} setting is deprecated. Migrate to "
                "MAILERS before Django 7.0."
            )
            with (
                self.subTest(name=name),
                override_deprecated_email_settings(**{name: "foo"}),
                self.assertWarnsMessage(RemovedInDjango70Warning, msg),
            ):
                getattr(django.conf.settings, name)

    @override_settings(MAILERS={})
    def test_deprecated_settings_do_not_exist_when_mailers_defined(self):
        # The global_settings defaults for the deprecated settings are hidden
        # when MAILERS is defined.
        for name in self.deprecated_settings:
            with self.subTest(name=name):
                self.assertFalse(hasattr(django.conf.settings, name))

    @override_settings(MAILERS={})
    def test_deprecated_settings_not_in_dir_when_mailers_defined(self):
        known_settings = dir(django.conf.settings)
        actual_overlap = set(self.deprecated_settings) & set(known_settings)
        self.assertEqual(actual_overlap, set())

    def test_deprecated_settings_are_in_dir_without_mailers(self):
        expected_settings = self.deprecated_setting_defaults.keys()
        known_settings = dir(django.conf.settings)
        actual_overlap = set(expected_settings) & set(known_settings)
        self.assertEqual(actual_overlap, expected_settings)

    @override_settings(MAILERS={})
    def test_error_when_using_deprecated_settings_with_mailers_defined(self):
        for name in self.deprecated_settings:
            msg = f"The {name} setting is not available when MAILERS is defined."
            with (
                self.subTest(name=name),
                override_deprecated_email_settings(**{name: "foo"}),
            ):
                with self.assertRaisesMessage(AttributeError, msg):
                    getattr(django.conf.settings, name)

    @ignore_warnings(category=RemovedInDjango70Warning)
    def test_direct_settings_manipulation(self):
        settings = self.init_simulated_settings()

        # Accessing deprecated setting (from global defaults) caches its value
        # in LazySettings.__dir__.
        self.assertEqual(settings.EMAIL_PORT, 25)

        # Defining MAILERS invalidates the cache but is not an error.
        settings.MAILERS = {}

        # Accessing the conflicting setting _is_ an error.
        msg = "not available when MAILERS is defined"
        with self.assertRaisesMessage(AttributeError, msg):
            _ = settings.EMAIL_PORT  # Not `25` from the cache.

        # Adding a conflicting setting is not an error, but accessing it is.
        settings.EMAIL_HOST = "example.com"
        with self.assertRaisesMessage(AttributeError, msg):
            _ = settings.EMAIL_HOST

        # Deleting MAILERS removes the conflict and allows access to
        # the deprecated settings again.
        del settings.MAILERS
        self.assertEqual(settings.EMAIL_PORT, 25)
        self.assertEqual(settings.EMAIL_HOST, "example.com")

    @override_settings(MAILERS={})
    def test_error_when_using_conflicting_setting_via_override_settings(self):
        # Overriding EMAIL_HOST when MAILERS is defined creates a
        # conflict but does not cause an immediate error.
        with override_deprecated_email_settings(EMAIL_HOST="example.com"):
            self.assertEqual(django.conf.settings.MAILERS, {})
            # Trying to access the conflicting setting causes an error.
            with self.assertRaisesMessage(AttributeError, "not available"):
                _ = django.conf.settings.EMAIL_HOST

            # Both conflicting settings are defined (neither is inherited from
            # default global_settings), so LazySettings.__dir__() does not hide
            # them.
            self.assertIn("EMAIL_HOST", dir(django.conf.settings))
            self.assertIn("MAILERS", dir(django.conf.settings))

            del django.conf.settings.EMAIL_HOST
            self.assertNotIn("EMAIL_HOST", dir(django.conf.settings))

    @ignore_warnings(category=RemovedInDjango70Warning)
    def test_deprecated_settings_defaults_unchanged(self):
        # Django's test runner overrides EMAIL_BACKEND in django.conf.settings,
        # so construct a fresh settings object for this test.
        settings = self.init_simulated_settings()
        for name, expected in self.deprecated_setting_defaults.items():
            with self.subTest(name=name):
                actual = getattr(settings, name)
                if expected is None:
                    self.assertIsNone(actual)
                elif expected is True or expected is False:
                    self.assertIs(actual, expected)
                else:
                    self.assertEqual(actual, expected)

    @ignore_warnings(category=RemovedInDjango70Warning)
    def test_email_backend_override_during_tests(self):
        self.assertEqual(
            django.conf.settings.EMAIL_BACKEND,
            "django.core.mail.backends.locmem.EmailBackend",
        )



import math
import os
import sys
from datetime import datetime

from django.core.exceptions import SuspiciousOperation
from django.core.serializers.json import DjangoJSONEncoder
from django.test import SimpleTestCase
from django.test.utils import override_settings
from django.utils.deprecation import RemovedInDjango70Warning
from django.utils.functional import lazystr
from django.utils.html import (
    conditional_escape,
    escape,
    escapejs,
    format_html,
    format_html_join,
    html_safe,
    json_script,
    linebreaks,
    smart_urlquote,
    strip_spaces_between_tags,
    strip_tags,
    urlize,
)
from django.utils.safestring import mark_safe


@override_settings(URLIZE_ASSUME_HTTPS=True)
class TestUtilsHtml(SimpleTestCase):
    def check_output(self, function, value, output=None):
        """
        function(value) equals output. If output is None, function(value)
        equals value.
        """
        if output is None:
            output = value
        self.assertEqual(function(value), output)

    def test_escape(self):
        items = (
            ("&", "&amp;"),
            ("<", "&lt;"),
            (">", "&gt;"),
            ('"', "&quot;"),
            ("'", "&#x27;"),
        )
        # Substitution patterns for testing the above items.
        patterns = ("%s", "asdf%sfdsa", "%s1", "1%sb")
        for value, output in items:
            with self.subTest(value=value, output=output):
                for pattern in patterns:
                    with self.subTest(value=value, output=output, pattern=pattern):
                        self.check_output(escape, pattern % value, pattern % output)
                        self.check_output(
                            escape, lazystr(pattern % value), pattern % output
                        )
                # Check repeated values.
                self.check_output(escape, value * 2, output * 2)
        # Verify it doesn't double replace &.
        self.check_output(escape, "<&", "&lt;&amp;")

    def test_format_html(self):
        self.assertEqual(
            format_html(
                "{} {} {third} {fourth}",
                "< Dangerous >",
                mark_safe("<b>safe</b>"),
                third="< dangerous again",
                fourth=mark_safe("<i>safe again</i>"),
            ),
            "&lt; Dangerous &gt; <b>safe</b> &lt; dangerous again <i>safe again</i>",
        )

    def test_format_html_no_params(self):
        msg = "args or kwargs must be provided."
        with self.assertRaisesMessage(TypeError, msg):
            name = "Adam"
            self.assertEqual(format_html(f"<i>{name}</i>"), "<i>Adam</i>")

    def test_format_html_join_with_positional_arguments(self):
        self.assertEqual(
            format_html_join(
                "\n",
                "<li>{}) {}</li>",
                [(1, "Emma"), (2, "Matilda")],
            ),
            "<li>1) Emma</li>\n<li>2) Matilda</li>",
        )

    def test_format_html_join_with_keyword_arguments(self):
        self.assertEqual(
            format_html_join(
                "\n",
                "<li>{id}) {text}</li>",
                [{"id": 1, "text": "Emma"}, {"id": 2, "text": "Matilda"}],
            ),
            "<li>1) Emma</li>\n<li>2) Matilda</li>",
        )

    def test_linebreaks(self):
        items = (
            ("para1\n\npara2\r\rpara3", "<p>para1</p>\n\n<p>para2</p>\n\n<p>para3</p>"),
            (
                "para1\nsub1\rsub2\n\npara2",
                "<p>para1<br>sub1<br>sub2</p>\n\n<p>para2</p>",
            ),
            (
                "para1\r\n\r\npara2\rsub1\r\rpara4",
                "<p>para1</p>\n\n<p>para2<br>sub1</p>\n\n<p>para4</p>",
            ),
            ("para1\tmore\n\npara2", "<p>para1\tmore</p>\n\n<p>para2</p>"),
        )
        for value, output in items:
            with self.subTest(value=value, output=output):
                self.check_output(linebreaks, value, output)
                self.check_output(linebreaks, lazystr(value), output)

    def test_strip_tags(self):
        # Python fixed a quadratic-time issue in HTMLParser in 3.13.6, 3.12.12.
        # The fix slightly changes HTMLParser's output, so tests for
        # particularly malformed input must handle both old and new results.
        # The check below is temporary until all supported Python versions and
        # CI workers include the fix. See:
        # https://github.com/python/cpython/commit/6eb6c5db
        min_fixed_security = {
            (3, 13): (3, 13, 6),
            (3, 12): (3, 12, 12),
        }
        # Similarly, there was a fix for terminating incomplete entities. See:
        # https://github.com/python/cpython/commit/95296a9d
        min_fixed_incomplete_entities = {
            (3, 14): (3, 14, 1),
            (3, 13): (3, 13, 10),
            (3, 12): (3, 12, math.inf),  # not fixed in 3.12.
        }
        major_version = sys.version_info[:2]
        htmlparser_fixed_security = sys.version_info >= min_fixed_security.get(
            major_version, major_version
        )
        htmlparser_fixed_incomplete_entities = (
            sys.version_info
            >= min_fixed_incomplete_entities.get(major_version, major_version)
        )
        items = (
            (
                "<p>See: &#39;&eacute; is an apostrophe followed by e acute</p>",
                "See: &#39;&eacute; is an apostrophe followed by e acute",
            ),
            (
                "<p>See: &#x27;&eacute; is an apostrophe followed by e acute</p>",
                "See: &#x27;&eacute; is an apostrophe followed by e acute",
            ),
            ("<adf>a", "a"),
            ("</adf>a", "a"),
            ("<asdf><asdf>e", "e"),
            ("hi, <f x", "hi, <f x"),
            ("234<235, right?", "234<235, right?"),
            ("a4<a5 right?", "a4<a5 right?"),
            ("b7>b2!", "b7>b2!"),
            ("</fe", "</fe"),
            ("<x>b<y>", "b"),
            ("a<p onclick=\"alert('<test>')\">b</p>c", "abc"),
            ("a<p a >b</p>c", "abc"),
            ("d<a:b c:d>e</p>f", "def"),
            ('<strong>foo</strong><a href="http://example.com">bar</a>', "foobar"),
            # caused infinite loop on Pythons not patched with
            # https://bugs.python.org/issue20288
            ("&gotcha&#;<>", "&gotcha&#;<>"),
            ("<sc<!-- -->ript>test<<!-- -->/script>", "ript>test"),
            (
                "<script>alert()</script>&h",
                "alert()&h;" if htmlparser_fixed_incomplete_entities else "alert()h",
            ),
            (
                "><!" + ("&" * 16000) + "D",
                ">" if htmlparser_fixed_security else "><!" + ("&" * 16000) + "D",
            ),
            ("X<<<<br>br>br>br>X", "XX"),
            ("<" * 50 + "a>" * 50, ""),
            (
                ">" + "<a" * 500 + "a",
                ">" if htmlparser_fixed_security else ">" + "<a" * 500 + "a",
            ),
            ("<a" * 49 + "a" * 951, "<a" * 49 + "a" * 951),
            ("<" + "a" * 1_002, "<" + "a" * 1_002),
        )
        for value, output in items:
            with self.subTest(value=value, output=output):
                self.check_output(strip_tags, value, output)
                self.check_output(strip_tags, lazystr(value), output)

    def test_strip_tags_suspicious_operation_max_depth(self):
        value = "<" * 51 + "a>" * 51, "<a>"
        with self.assertRaises(SuspiciousOperation):
            strip_tags(value)

    def test_strip_tags_suspicious_operation_large_open_tags(self):
        items = [
            ">" + "<a" * 501,
            "<a" * 50 + "a" * 950,
        ]
        for value in items:
            with self.subTest(value=value):
                with self.assertRaises(SuspiciousOperation):
                    strip_tags(value)

    def test_strip_tags_files(self):
        # Test with more lengthy content (also catching performance
        # regressions)
        for filename in ("strip_tags1.html", "strip_tags2.txt"):
            with self.subTest(filename=filename):
                path = os.path.join(os.path.dirname(__file__), "files", filename)
                with open(path) as fp:
                    content = fp.read()
                    start = datetime.now()
                    stripped = strip_tags(content)
                    elapsed = datetime.now() - start
                self.assertEqual(elapsed.seconds, 0)
                self.assertIn("Test string that has not been stripped.", stripped)
                self.assertNotIn("<", stripped)

    def test_strip_spaces_between_tags(self):
        # Strings that should come out untouched.
        items = (" <adf>", "<adf> ", " </adf> ", " <f> x</f>")
        for value in items:
            with self.subTest(value=value):
                self.check_output(strip_spaces_between_tags, value)
                self.check_output(strip_spaces_between_tags, lazystr(value))

        # Strings that have spaces to strip.
        items = (
            ("<d> </d>", "<d></d>"),
            ("<p>hello </p>\n<p> world</p>", "<p>hello </p><p> world</p>"),
            ("\n<p>\t</p>\n<p> </p>\n", "\n<p></p><p></p>\n"),
        )
        for value, output in items:
            with self.subTest(value=value, output=output):
                self.check_output(strip_spaces_between_tags, value, output)
                self.check_output(strip_spaces_between_tags, lazystr(value), output)

    def test_escapejs(self):
        items = (
            (
                "\"double quotes\" and 'single quotes'",
                "\\u0022double quotes\\u0022 and \\u0027single quotes\\u0027",
            ),
            (r"\ : backslashes, too", "\\u005C : backslashes, too"),
            (
                "and lots of whitespace: \r\n\t\v\f\b",
                "and lots of whitespace: \\u000D\\u000A\\u0009\\u000B\\u000C\\u0008",
            ),
            (
                r"<script>and this</script>",
                "\\u003Cscript\\u003Eand this\\u003C/script\\u003E",
            ),
            (
                "paragraph separator:\u2029and line separator:\u2028",
                "paragraph separator:\\u2029and line separator:\\u2028",
            ),
            ("`", "\\u0060"),
            ("\u007f", "\\u007F"),
            ("\u0080", "\\u0080"),
            ("\u009f", "\\u009F"),
        )
        for value, output in items:
            with self.subTest(value=value, output=output):
                self.check_output(escapejs, value, output)
                self.check_output(escapejs, lazystr(value), output)

    def test_json_script(self):
        tests = (
            # "<", ">" and "&" are quoted inside JSON strings
            (
                (
                    "&<>",
                    '<script id="test_id" type="application/json">'
                    '"\\u0026\\u003C\\u003E"</script>',
                )
            ),
            # "<", ">" and "&" are quoted inside JSON objects
            (
                {"a": "<script>test&ing</script>"},
                '<script id="test_id" type="application/json">'
                '{"a": "\\u003Cscript\\u003Etest\\u0026ing\\u003C/script\\u003E"}'
                "</script>",
            ),
            # Lazy strings are quoted
            (
                lazystr("&<>"),
                '<script id="test_id" type="application/json">"\\u0026\\u003C\\u003E"'
                "</script>",
            ),
            (
                {"a": lazystr("<script>test&ing</script>")},
                '<script id="test_id" type="application/json">'
                '{"a": "\\u003Cscript\\u003Etest\\u0026ing\\u003C/script\\u003E"}'
                "</script>",
            ),
        )
        for arg, expected in tests:
            with self.subTest(arg=arg):
                self.assertEqual(json_script(arg, "test_id"), expected)

    def test_json_script_custom_encoder(self):
        class CustomDjangoJSONEncoder(DjangoJSONEncoder):
            def encode(self, o):
                return '{"hello": "world"}'

        self.assertHTMLEqual(
            json_script({}, encoder=CustomDjangoJSONEncoder),
            '<script type="application/json">{"hello": "world"}</script>',
        )

    def test_json_script_without_id(self):
        self.assertHTMLEqual(
            json_script({"key": "value"}),
            '<script type="application/json">{"key": "value"}</script>',
        )

    def test_smart_urlquote(self):
        items = (
            # IDN is encoded as percent-encoded ("quoted") UTF-8 (#36013).
            ("http://öäü.com/", "http://%C3%B6%C3%A4%C3%BC.com/"),
            ("https://faß.example.com", "https://fa%C3%9F.example.com"),
            (
                "http://öäü.com/öäü/",
                "http://%C3%B6%C3%A4%C3%BC.com/%C3%B6%C3%A4%C3%BC/",
            ),
            (
                # Valid under IDNA 2008, but was invalid in IDNA 2003.
                "https://މިހާރު.com",
                "https://%DE%89%DE%A8%DE%80%DE%A7%DE%83%DE%AA.com",
            ),
            (
                # Valid under WHATWG URL Specification but not IDNA 2008.
                "http://👓.ws",
                "http://%F0%9F%91%93.ws",
            ),
            # Pre-encoded IDNA is left unchanged.
            ("http://xn--iny-zx5a.com/idna2003", "http://xn--iny-zx5a.com/idna2003"),
            ("http://xn--fa-hia.com/idna2008", "http://xn--fa-hia.com/idna2008"),
            # Everything unsafe is quoted, !*'();:@&=+$,/?#[]~ is considered
            # safe as per RFC.
            (
                "http://example.com/path/öäü/",
                "http://example.com/path/%C3%B6%C3%A4%C3%BC/",
            ),
            ("http://example.com/%C3%B6/ä/", "http://example.com/%C3%B6/%C3%A4/"),
            ("http://example.com/?x=1&y=2+3&z=", "http://example.com/?x=1&y=2+3&z="),
            ("http://example.com/?x=<>\"'", "http://example.com/?x=%3C%3E%22%27"),
            (
                "http://example.com/?q=http://example.com/?x=1%26q=django",
                "http://example.com/?q=http%3A%2F%2Fexample.com%2F%3Fx%3D1%26q%3D"
                "django",
            ),
            (
                "http://example.com/?q=http%3A%2F%2Fexample.com%2F%3Fx%3D1%26q%3D"
                "django",
                "http://example.com/?q=http%3A%2F%2Fexample.com%2F%3Fx%3D1%26q%3D"
                "django",
            ),
            ("http://.www.f oo.bar/", "http://.www.f%20oo.bar/"),
            ('http://example.com">', "http://example.com%22%3E"),
            ("http://10.22.1.1/", "http://10.22.1.1/"),
            ("http://[fd00::1]/", "http://[fd00::1]/"),
        )
        for value, output in items:
            with self.subTest(value=value, output=output):
                self.assertEqual(smart_urlquote(value), output)

    def test_conditional_escape(self):
        s = "<h1>interop</h1>"
        self.assertEqual(conditional_escape(s), "&lt;h1&gt;interop&lt;/h1&gt;")
        self.assertEqual(conditional_escape(mark_safe(s)), s)
        self.assertEqual(conditional_escape(lazystr(mark_safe(s))), s)

    def test_html_safe(self):
        @html_safe
        class HtmlClass:
            def __str__(self):
                return "<h1>I'm a html class!</h1>"

        html_obj = HtmlClass()
        self.assertTrue(hasattr(HtmlClass, "__html__"))
        self.assertTrue(hasattr(html_obj, "__html__"))
        self.assertEqual(str(html_obj), html_obj.__html__())

    def test_html_safe_subclass(self):
        class BaseClass:
            def __html__(self):
                # defines __html__ on its own
                return "some html content"

            def __str__(self):
                return "some non html content"

        @html_safe
        class Subclass(BaseClass):
            def __str__(self):
                # overrides __str__ and is marked as html_safe
                return "some html safe content"

        subclass_obj = Subclass()
        self.assertEqual(str(subclass_obj), subclass_obj.__html__())

    def test_html_safe_defines_html_error(self):
        msg = "can't apply @html_safe to HtmlClass because it defines __html__()."
        with self.assertRaisesMessage(ValueError, msg):

            @html_safe
            class HtmlClass:
                def __html__(self):
                    return "<h1>I'm a html class!</h1>"

    def test_html_safe_doesnt_define_str(self):
        msg = "can't apply @html_safe to HtmlClass because it doesn't define __str__()."
        with self.assertRaisesMessage(ValueError, msg):

            @html_safe
            class HtmlClass:
                pass

    def test_urlize(self):
        tests = (
            (
                "Search for google.com/?q=! and see.",
                'Search for <a href="https://google.com/?q=">google.com/?q=</a>! and '
                "see.",
            ),
            (
                "Search for google.com/?q=1&lt! and see.",
                'Search for <a href="https://google.com/?q=1%3C">google.com/?q=1&lt'
                "</a>! and see.",
            ),
            (
                lazystr("Search for google.com/?q=!"),
                'Search for <a href="https://google.com/?q=">google.com/?q=</a>!',
            ),
            (
                "http://www.foo.bar/",
                '<a href="http://www.foo.bar/">http://www.foo.bar/</a>',
            ),
            (
                "Look on www.نامه‌ای.com.",
                "Look on <a "
                'href="https://www.%D9%86%D8%A7%D9%85%D9%87%E2%80%8C%D8%A7%DB%8C.com"'
                ">www.نامه‌ای.com</a>.",
            ),
            ("foo@example.com", '<a href="mailto:foo@example.com">foo@example.com</a>'),
            (
                "test@" + "한.글." * 15 + "aaa",
                '<a href="mailto:test@'
                + "%ED%95%9C.%EA%B8%80." * 15
                + 'aaa">'
                + "test@"
                + "한.글." * 15
                + "aaa</a>",
            ),
            (
                # RFC 6068 requires a mailto URI to percent-encode a number of
                # characters that can appear in <addr-spec>.
                "yes+this=is&a%valid!email@example.com",
                '<a href="mailto:yes%2Bthis%3Dis%26a%25valid%21email@example.com"'
                ">yes+this=is&a%valid!email@example.com</a>",
            ),
            (
                "foo@faß.example.com",
                '<a href="mailto:foo@fa%C3%9F.example.com">foo@faß.example.com</a>',
            ),
            (
                "idna-2008@މިހާރު.example.mv",
                '<a href="mailto:idna-2008@%DE%89%DE%A8%DE%80%DE%A7%DE%83%DE%AA.ex'
                'ample.mv">idna-2008@މިހާރު.example.mv</a>',
            ),
            (
                "host.djangoproject.com",
                '<a href="https://host.djangoproject.com">host.djangoproject.com</a>',
            ),
        )
        for value, output in tests:
            with self.subTest(value=value):
                self.assertEqual(urlize(value), output)

    @override_settings(URLIZE_ASSUME_HTTPS=False)
    def test_urlize_http_default_warning(self):
        msg = (
            "The default protocol will be changed from HTTP to HTTPS in Django 7.0. "
            "Set the URLIZE_ASSUME_HTTPS transitional setting to True to opt into "
            "using HTTPS as the new default protocol."
        )
        with self.assertWarnsMessage(RemovedInDjango70Warning, msg):
            self.assertEqual(
                urlize("Visit example.com"),
                'Visit <a href="http://example.com">example.com</a>',
            )

    def test_urlize_unchanged_inputs(self):
        tests = (
            ("a" + "@a" * 50000) + "a",  # simple_email_re catastrophic test
            # Unicode domain catastrophic tests.
            "a@" + "한.글." * 1_000_000 + "a",
            "http://" + "한.글." * 1_000_000 + "com",
            "www." + "한.글." * 1_000_000 + "com",
            ("a" + "." * 1000000) + "a",  # trailing_punctuation catastrophic test
            "foo@",
            "@foo.com",
            "foo@.example.com",
            "foo@localhost",
            "foo@localhost.",
            "test@example?;+!.com",
            "email me@example.com,then I'll respond",
            "[a link](https://www.djangoproject.com/)",
            # trim_punctuation catastrophic tests
            "(" * 100_000 + ":" + ")" * 100_000,
            "(" * 100_000 + "&:" + ")" * 100_000,
            "([" * 100_000 + ":" + "])" * 100_000,
            "[(" * 100_000 + ":" + ")]" * 100_000,
            "([[" * 100_000 + ":" + "]])" * 100_000,
            "&:" + ";" * 100_000,
            "&.;" * 100_000,
            ".;" * 100_000,
            "&" + ";:" * 100_000,
        )
        for value in tests:
            with self.subTest(value=value):
                self.assertEqual(urlize(value), value)



from pathlib import Path

from django.http import FileResponse


def template_bad_filename(request):
    content = Path(__file__).parent / "custom_templates" / "project_template.tgz"
    f = open(content, "rb")
    filename = "/nonexistent/archive.tgz"
    response = FileResponse(
        f,
        as_attachment=True,
        filename=filename,
    )
    # Force the filename to have a slash at the beginning.
    response["Content-Disposition"] = f'attachment; filename="{filename}"'
    return response



from django.http import HttpResponse
from django.views import View


def some_view(request):
    return HttpResponse("ok")


def params_view(request, slug):
    return HttpResponse(f"Params: {slug}")


class SomeView(View):
    def get(self, request):
        return HttpResponse("ok")


class ParamsView(View):
    def get(self, request, pk):
        return HttpResponse(f"Params: {pk}")


some_cbv = SomeView.as_view()
params_cbv = ParamsView.as_view()
